# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""

import dash_bootstrap_components as dbc

from ansys.saf.glow.client.dashclient import DashClient
from dash_extensions.enrich import Input, Output, State, callback, dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{cookiecutter.__solution_definition_name}}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.problem_setup_step import ProblemSetupStep
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.monitoring_tabs import (
    design_table_page,
    project_summary_page,
    result_files_page,
    scenery_page,
    status_overview_page,
    summary_page,
    visualization_page,
)


def layout(monitoring_step: MonitoringStep, problem_setup_step: ProblemSetupStep):

    if not len(monitoring_step.available_root_nodes) and problem_setup_step.optislang_solve_status != "initial":
        monitoring_step.get_system_hierarchy()

    return html.Div(
        [
            dbc.Row(
                [
                    dbc.Accordion(
                        [
                            dbc.AccordionItem(
                                [
                                    html.Div(
                                        [
                                            html.Div(html.H5(children="""Root""")),
                                            dcc.Dropdown(
                                                monitoring_step.available_root_nodes,
                                                None,
                                                id="root_level",
                                                multi=False,
                                                clearable=True,
                                                disabled=False,
                                            ),
                                            dcc.Loading(
                                                type="circle",
                                                fullscreen=True,
                                                color="#ffb71b",
                                                style={
                                                    "background-color": "rgba(55, 58, 54, 0.1)",
                                                },
                                                children=html.Div(id="wait_system_update"),
                                            ),
                                        ],
                                        style={"width": "20%", "display": "inline-block"},
                                    ),
                                    html.Div(
                                        [
                                            html.Div(html.H5(children="""System""")),
                                            dcc.Dropdown(
                                                id="system_level",
                                                multi=False,
                                                clearable=True,
                                                disabled=False,
                                            ),
                                            dcc.Loading(
                                                type="circle",
                                                fullscreen=True,
                                                color="#ffb71b",
                                                style={
                                                    "background-color": "rgba(55, 58, 54, 0.1)",
                                                },
                                                children=html.Div(id="wait_subsystem_update"),
                                            ),
                                        ],
                                        style={"width": "20%", "display": "inline-block"},
                                    ),
                                    html.Div(
                                        [
                                            html.Div(html.H5(children="""Sub-System""")),
                                            dcc.Dropdown(
                                                id="subsystem_level",
                                                multi=False,
                                                clearable=True,
                                                disabled=False,
                                            ),
                                        ],
                                        style={"width": "20%", "display": "inline-block"},
                                    ),
                                ],
                                title="Node selection",
                                item_id="node-selection-accordion-item",
                            ),
                            dbc.AccordionItem(
                                [
                                    dbc.Tabs(
                                        id="monitoring_tabs",
                                        active_tab="summary_tab",
                                        style={
                                            "color": "#000000",
                                            "background-color": "#FFFFFF",
                                            "text-color": "#000000",
                                        },
                                    ),
                                    html.Div(
                                        id="monitoring_page_content",
                                    ),
                                    dcc.Loading(
                                        type="circle",
                                        fullscreen=True,
                                        color="#ffb71b",
                                        style={
                                            "background-color": "rgba(55, 58, 54, 0.1)",
                                        },
                                        children=html.Div(id="wait_page_content_update"),
                                    ),
                                ],
                                title="Monitoring",
                                item_id="monitoring-accordion-item",
                            ),
                        ],
                        id="monitoring-accordion",
                        active_item=[
                            "node-selection-accordion-item",
                            "monitoring-accordion-item",
                        ],
                        always_open=True,
                    )
                ],
            ),
        ]
    )


@callback(
    Output("system_level", "options"),
    Output("wait_system_update", "children"),
    Input("root_level", "value"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def update_system_dropdown(root_node_name, pathname):
    """"""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    monitoring_step = project.steps.monitoring_step
    monitoring_step.selected_root_node = root_node_name
    return monitoring_step.available_system_nodes[root_node_name], True


@callback(
    Output("subsystem_level", "options"),
    Output("wait_subsystem_update", "children"),
    Input("system_level", "value"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def update_subsystem_dropdown(system_node_name, pathname):
    """"""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    monitoring_step = project.steps.monitoring_step
    monitoring_step.selected_system_node = system_node_name
    return monitoring_step.available_subsystem_nodes[system_node_name], True


@callback(
    Output("monitoring_tabs", "children"),
    Input("root_level", "value"),
    Input("system_level", "value"),
    Input("subsystem_level", "value"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def update_tabs(root_node_name, system_node_name, subsystem_node_name, pathname):
    """"""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    monitoring_step = project.steps.monitoring_step

    monitoring_step.selected_root_node = root_node_name
    monitoring_step.selected_system_node = system_node_name
    monitoring_step.selected_subsystem_node = subsystem_node_name

    monitoring_step.get_node_kind()

    if monitoring_step.selected_node_kind == "root":
        return [
            dbc.Tab(
                label="Project Summary",
                tab_id="project_summary_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Result Files",
                tab_id="result_files_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Scenery",
                tab_id="scenery_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Design Table",
                tab_id="design_table_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Status Overview",
                tab_id="status_overview_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
        ]
    elif monitoring_step.selected_node_kind == "system":
        return [
            dbc.Tab(
                label="Summary",
                tab_id="summary_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Design Table",
                tab_id="design_table_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Visualization",
                tab_id="visualization_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Status Overview",
                tab_id="status_overview_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
        ]
    elif monitoring_step.selected_node_kind == "actor":
        return [
            dbc.Tab(
                label="Summary",
                tab_id="summary_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
            dbc.Tab(
                label="Status Overview",
                tab_id="status_overview_tab",
                label_style={
                    "color": "#000000",
                    "text-color": "#000000",
                },
                active_label_style={
                    "color": "#FFFFFF",
                    "text-color": "#000000",
                    "background-color": "#000000",
                    "border-style": "solid",
                    "border-color": "#000000",
                },
            ),
        ]


@callback(
    Output("monitoring_page_content", "children"),
    Output("wait_page_content_update", "children"),
    Input("monitoring_tabs", "active_tab"),
    Input("url", "pathname"),
    prevent_initial_call=True,
)
def display_monitoring_page(active_tab, pathname):
    """Display monitoring page content that corresponds to the tab selected."""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)

    if active_tab == "project_summary_tab":
        return project_summary_page.layout(project.steps.monitoring_step), True
    elif active_tab == "summary_tab":
        return summary_page.layout(project.steps.monitoring_step), True
    elif active_tab == "result_files_tab":
        return result_files_page.layout(project.steps.monitoring_step), True
    elif active_tab == "scenery_tab":
        return scenery_page.layout(project.steps.monitoring_step), True
    elif active_tab == "design_table_tab":
        return design_table_page.layout(project.steps.monitoring_step), True
    elif active_tab == "visualization_tab":
        return visualization_page.layout(project.steps.monitoring_step), True
    elif active_tab == "status_overview_tab":
        return status_overview_page.layout(project.steps.monitoring_step), True



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""

from ansys.solutions.dash_components.table import OutputRow
import dash_bootstrap_components as dbc
from dash_extensions.enrich import dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep


def layout(monitoring_step: MonitoringStep):
    """Layout of the second step UI."""

    monitoring_step.get_summary()
    font_size = "15px"

    return html.Div(
        [
            html.Br(),
            dbc.Row(
                [
                    dcc.Markdown("**Actor Information**", style={"font-size": "20px"}),
                    OutputRow(
                        "label",
                        "working_directory",
                        "Working directory",
                        row_default_value=monitoring_step.summary["working_directory"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "processing_state",
                        "Processing state",
                        row_default_value=monitoring_step.summary["state"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "execution_duration",
                        "Execution duration",
                        row_default_value=monitoring_step.summary["execution_duration"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "processed",
                        "Processed",
                        row_default_value=monitoring_step.summary["processed"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get()
                    if monitoring_step.summary["processed"]
                    else None,
                    OutputRow(
                        "label",
                        "status",
                        "Status",
                        row_default_value=monitoring_step.summary["status"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get()
                    if monitoring_step.summary["status"]
                    else None,
                    OutputRow(
                        "label",
                        "succeeded",
                        "Succeeded",
                        row_default_value=monitoring_step.summary["succeeded"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get()
                    if monitoring_step.summary["succeeded"]
                    else None,
                    OutputRow(
                        "label",
                        "not_succeeded",
                        "Not succeeded",
                        row_default_value=monitoring_step.summary["not_succeeded"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get()
                    if monitoring_step.summary["not_succeeded"]
                    else None,
                ],
            ),
        ]
    )



from ansys.templates.utils import keep_files


DESIRED_STRUCTURE = [
    ".github/workflows/ci.yml",
    ".vscode/launch.json",
    "doc/source/_static/ansys-solutions-logo-black-background.png",
    "doc/source/_static/README.md",
    "doc/source/_templates/README.md",
    "doc/source/conf.py",
    "doc/source/index.rst",
    "doc/styles/Vocab/ANSYS/accept.txt",
    "doc/styles/Vocab/ANSYS/reject.txt",
    "doc/styles/.gitignore",
    "doc/.vale.ini",
    "doc/make.bat",
    "doc/Makefile",
    "examples/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/scripts/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/definition.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/intro_step.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/first_step.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/second_step.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/__init__.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/main.py",
    "tests/common_test_files/README.md",
    "tests/integration/test_integration_dummy.py",
    "tests/unit/test_unit_dummy.py",
    "tests/conftest.py",
    ".codespell.exclude",
    ".codespell.ignore",
    ".flake8",
    ".gitignore",
    ".pre-commit-config.yaml",
    "CHANGELOG.md",
    "CODE_OF_CONDUCT.md",
    "CODEOWNERS",
    "CONTRIBUTING.md",
    "LICENSE.rst",
    "pyproject.toml",
    "README.rst",
    "setup_environment.py",
    "tox.ini",
    ".env"
]
"""A list holding all desired files to be included in the project."""

UI_STRUCTURE = [
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/assets/Graphics/ansys-solutions-horizontal-logo.png",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/assets/Graphics/solution-workflow-sketch.png",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/assets/style.css",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/app.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/intro_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/first_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/second_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/page.py",
]

# Add UI structure to desired structure if applicable
if "{{ cookiecutter.with_dash_ui }}" == "yes":
    DESIRED_STRUCTURE = DESIRED_STRUCTURE + UI_STRUCTURE

def main():
    """Entry point of the script."""
    # Apply the desired structure to the project
    keep_files(DESIRED_STRUCTURE)

if __name__ == "__main__":
    main()



"""{{ cookiecutter.__project_name_slug }}."""
{%- if cookiecutter.copyright != "None" %}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}

import os
import sys

from ._version import __version__

sys.path.append(os.path.join(os.path.dirname(__file__)))



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Initialization of the frontend layout across all the steps."""


from ansys.saf.glow.client.dashclient import DashClient
from ansys_dash_treeview import AnsysDashTreeview
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, callback, callback_context, dcc, html
from dash_iconify import DashIconify

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import (
    {{ cookiecutter.__solution_definition_name }},
)
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui import intro_page, first_page, second_page

step_list = [
    {
        "key": "intro_step",
        "text": "Introduction",
        "depth": 0,
    },
    {
        "key": "first_step",
        "text": "First Step",
        "depth": 0,
    },
    {
        "key": "second_step",
        "text": "Second Step",
        "depth": 0,
    }
]


layout = html.Div(
    [
        dcc.Location(id="url", refresh=False), # represents the browser address bar and doesn't render anything
        dbc.Stack(
            [
                html.Div(
                    [
                        html.Img(
                        src = r"/assets/Graphics/ansys-solutions-horizontal-logo.png",
                        style={'width': '80%'}
                    )
                    ],
                ),
                html.Div(
                    [
                        dbc.Button(
                            "Project Name:",
                            id = "project-name",
                            disabled = True,
                            style={
                                "color": "rgba(0, 0, 0, 1)",
                                "background-color": "rgba(255, 255, 255, 1)",
                                "border-color": "rgba(0, 0, 0, 1)"
                            },
                        )
                    ],
                    className="ms-auto",
                ),
                html.Div(id="return-to-portal"),
            ],
            direction = "horizontal",
            gap = 3,
        ),
        html.Br(),
        dbc.Row(
            [
                dbc.Col(
                    AnsysDashTreeview(
                        id="navigation_tree",
                        items=step_list,
                        children=[
                            DashIconify(icon="bi:caret-right-square-fill"),
                            DashIconify(icon="bi:caret-down-square-fill"),
                        ],
                        style={"showButtons": True, "focusColor": "#ffb71b", "itemHeight": "32"},  # Ansys gold
                    ),
                    width=2,
                    style={"background-color": "rgba(242, 242, 242, 0.6)"},  # Ansys grey
                ),
                dbc.Col(
                    html.Div(
                        id="page-content",
                        style={"padding-right": "1%"}
                    ),
                    width=10
                ),
            ],
        ),
    ]
)


@callback(
    Output("return-to-portal", "children"),
    Input("url", "pathname"),
)
def return_to_portal(pathname):
    """Display Solution Portal when back-to-portal button gets selected."""
    portal_ui_url = DashClient.get_portal_ui_url()
    children = (
        []
        if portal_ui_url is None
        else [
            dbc.Button(
                "Back to Projects",
                id = "return-to-portal",
                className = "me-2",
                n_clicks = 0,
                href = portal_ui_url,
                style = {"background-color": "rgba(0, 0, 0, 1)", "border-color": "rgba(0, 0, 0, 1)"},
            )
        ]
    )
    return children


@callback(
    Output("project-name", "children"),
    Input("url", "pathname"),
)
def display_poject_name(pathname):
    """Display current project name."""
    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    return f"Project Name: {project.project_display_name}"


# this callback is essential for initializing the step based on the persisted
# state of the project when the browser first displays the project to the user
# given the project's URL
@callback(
    Output("page-content", "children"),
    [
        Input("url", "pathname"),
        Input("navigation_tree", "focus"),
    ],
    prevent_initial_call=True,
)
def display_page(pathname, value):
    """Display page content."""
    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    triggered_id = callback_context.triggered[0]["prop_id"].split(".")[0]
    if triggered_id == "url":
        return intro_page.layout(project.steps.intro_step)
    if triggered_id == "navigation_tree":
        if value is None:
            page_layout = html.H1("Welcome!")
        elif value == "intro_step":
            page_layout = intro_page.layout(project.steps.intro_step)
        elif value == "first_step":
            page_layout = first_page.layout(project.steps.first_step)
        elif value == "second_step":
            page_layout = second_page.layout(project.steps.second_step)
        return page_layout



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the first step."""

import base64
import os
from pathlib import Path

import dash_bootstrap_components as dbc
from dash_extensions.enrich import dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.intro_step import IntroStep


def layout(step: IntroStep):
    """Layout of the first step UI."""

    solution_workflow_sketch_encoded = base64.b64encode(
        open(
            os.path.join(Path(__file__).parent.absolute(), "assets", "Graphics", "solution-workflow-sketch.png"), "rb"
        ).read()
    )

    return html.Div(
        dbc.Container(
            [
                html.H1("Add a title.", className="display-3", style={"font-size": "35px"}),
                html.P(
                    "Add a short sentence to describe the goal of the solution.",
                    className="lead",
                    style={"font-size": "20px"},
                ),
                html.Hr(className="my-2"),
                dbc.Row(
                    [
                        dbc.Col(
                            [
                                dbc.Row(
                                    [
                                        dcc.Markdown(
                                            """
                                            **Description**

                                            Information needed.
                                            """,
                                            className="lead",
                                            style={
                                                "textAlign": "left",
                                                "marginLeft": "auto",
                                                "marginRight": "auto",
                                                "font-size": "15px",
                                            },
                                        ),
                                        html.Br(),
                                        dcc.Markdown(
                                            """
                                            **Customer goals**

                                            Information needed.
                                            """,
                                            className="lead",
                                            style={
                                                "textAlign": "left",
                                                "font-size": "15px",
                                            },
                                        ),
                                        html.Br(),
                                        dcc.Markdown(
                                            """
                                            **Engineering goals**

                                            Information needed.
                                            """,
                                            className="lead",
                                            style={
                                                "textAlign": "left",
                                                "font-size": "15px",
                                            },
                                        ),
                                    ]
                                )
                            ],
                            width=6,
                        ),
                        dbc.Col(
                            [
                                dbc.Row(
                                    [
                                        dbc.Card(
                                            [
                                                dbc.CardImg(
                                                    src="data:image/png;base64,{}".format(
                                                        solution_workflow_sketch_encoded.decode()
                                                    ),
                                                ),
                                                dbc.CardFooter("This is the introduction image."),
                                            ],
                                            style={"width": "50rem"},
                                        )
                                    ]
                                )
                            ],
                            width=6,
                        ),
                    ]
                ),
            ],
            fluid=True,
            className="py-3",
        ),
        className="p-3 bg-light rounded-3",
    )



import shutil
import os
from pathlib import Path
from ansys.templates.utils import keep_files


DESIRED_STRUCTURE = [
    ".github/workflows/ci.yml",
    ".vscode/launch.json",
    "doc/source/_static/ansys-solutions-logo-black-background.png",
    "doc/source/_static/README.md",
    "doc/source/_templates/README.md",
    "doc/source/conf.py",
    "doc/source/index.rst",
    "doc/styles/Vocab/ANSYS/accept.txt",
    "doc/styles/Vocab/ANSYS/reject.txt",
    "doc/styles/.gitignore",
    "doc/.vale.ini",
    "doc/make.bat",
    "doc/Makefile",
    "examples/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/assets/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/scripts/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/README.md",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/utils.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/definition.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/monitoring_step.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/solution/problem_setup_step.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/assets/Graphics/ansys-solutions-horizontal-logo.png",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/assets/style.css",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/design_table_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/project_summary_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/result_files_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/scenery_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/status_overview_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/summary_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_tabs/visualization_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/app.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/colorscale.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/monitoring_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/problem_setup_page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/ui/page.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/__init__.py",
    f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/main.py",
    "tests/common_test_files/README.md",
    "tests/integration/test_integration_dummy.py",
    "tests/unit/test_unit_dummy.py",
    "tests/conftest.py",
    ".codespell.exclude",
    ".codespell.ignore",
    ".flake8",
    ".gitignore",
    ".pre-commit-config.yaml",
    "CHANGELOG.md",
    "CODE_OF_CONDUCT.md",
    "CODEOWNERS",
    "CONTRIBUTING.md",
    "LICENSE.rst",
    "pyproject.toml",
    "README.rst",
    "setup_environment.py",
    "tox.ini"
]
"""A list holding all desired files to be included in the project."""

ASSETS_DIRCTORY = Path(f"src/ansys/solutions/{{ cookiecutter.__solution_name_slug }}/model/assets/").absolute()


def copy_file_to_assets_folder(file_path: str, destination: str):
    """Copy a file if it exists."""

    if not os.path.isabs(file_path):
        working_directory = os.path.dirname(os.getcwd())
        file_path = os.path.join(working_directory, file_path)

    if os.path.exists(file_path):
        shutil.copy(file_path, destination)
    else:
        print(f"Unable to find {file_path}.")
        # raise FileNotFoundError(f"Unable to find {file_path}.")


def main():
    """Entry point of the script."""

    keep_files(DESIRED_STRUCTURE)

    if len("{{ cookiecutter.__optiSLang_project_file }}".replace(" ", "")):
        copy_file_to_assets_folder("{{ cookiecutter.__optiSLang_project_file }}",  str(ASSETS_DIRCTORY / "{{ cookiecutter.__optiSLang_project_file_name }}"))
    if len("{{ cookiecutter.__optiSLang_properties_file }}".replace(" ", "")):
        copy_file_to_assets_folder("{{ cookiecutter.__optiSLang_properties_file }}", str(ASSETS_DIRCTORY / "{{ cookiecutter.__optiSLang_properties_file_name }}"))


if __name__ == "__main__":
    main()



# Copyright (C) 2023 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""A collection of useful paths pointing towards each available template."""

import os
from pathlib import Path

_PATHS_MODULE = Path(os.path.dirname(os.path.abspath(__file__)))

LICENSES_TEMPLATES_PATH = _PATHS_MODULE / "licenses"
"""Path to the software licenses templates."""

PYTHON_TEMPLATES_PATH = _PATHS_MODULE / "python"
"""Path to the Python templates."""

PYTHON_TEMPLATES_COMMON_PATH = PYTHON_TEMPLATES_PATH / "common"
"""Path to the Python common template."""

PYTHON_TEMPLATES_DOC_PROJECT = PYTHON_TEMPLATES_PATH / "doc_project"
"""Path to the documentation project template."""

PYTHON_TEMPLATES_PYBASIC_PATH = PYTHON_TEMPLATES_PATH / "pybasic"
"""Path to the basic Python Package template."""

PYTHON_TEMPLATES_PYANSYS_PATH = PYTHON_TEMPLATES_PATH / "pyansys"
"""Path to the basic PyAnsys Python Package template."""

PYTHON_TEMPLATES_PYANSYS_ADVANCED_PATH = PYTHON_TEMPLATES_PATH / "pyansys_advanced"
"""Path to the advanced PyAnsys Python Package template."""

PYTHON_TEMPLATES_PYANSYS_OPENAPI_CLIENT_PATH = PYTHON_TEMPLATES_PATH / "pyansys_openapi_client"
"""Path to the PyAnsys OpenAPI Client Package template."""

PYTHON_TEMPLATES_PYACE_PATH = PYTHON_TEMPLATES_PATH / "pyace_pkg"
"""Path to the classic Python Project template."""

PYTHON_TEMPLATES_PYACE_GRPC_PATH = PYTHON_TEMPLATES_PATH / "pyace_grpc"
"""Path to the gRPC based Python Project template."""

PYTHON_TEMPLATES_PYACE_FLASK_PATH = PYTHON_TEMPLATES_PATH / "pyace_flask"
"""Path to the Flask based Python Project template."""

PYTHON_TEMPLATES_PYACE_FAST_PATH = PYTHON_TEMPLATES_PATH / "pyace_fastapi"
"""Path to the FastAPI based Python Project template."""

PYTHON_TEMPLATES_SOLUTION_PATH = PYTHON_TEMPLATES_PATH / "solution"
"""Path to the Solution template."""

PYTHON_TEMPLATES_OSL_SOLUTION_PATH = PYTHON_TEMPLATES_PATH / "osl_solution"
"""Path to the optiSLang Solution template."""

PYTHON_TEMPLATES_OSL_SOLUTION_FROM_OWA_PATH = PYTHON_TEMPLATES_PATH / "osl_solution_from_owa"
"""Path to the optiSLang Solution template from owa file."""

TEMPLATE_PATH_FINDER = {
    "common": PYTHON_TEMPLATES_COMMON_PATH,
    "doc-project": PYTHON_TEMPLATES_DOC_PROJECT,
    "osl-solution": PYTHON_TEMPLATES_OSL_SOLUTION_PATH,
    "pybasic": PYTHON_TEMPLATES_PYBASIC_PATH,
    "pyansys": PYTHON_TEMPLATES_PYANSYS_PATH,
    "pyansys-advanced": PYTHON_TEMPLATES_PYANSYS_ADVANCED_PATH,
    "pyansys-openapi-client": PYTHON_TEMPLATES_PYANSYS_OPENAPI_CLIENT_PATH,
    "pyace": PYTHON_TEMPLATES_PYACE_PATH,
    "pyace-grpc": PYTHON_TEMPLATES_PYACE_GRPC_PATH,
    "pyace-flask": PYTHON_TEMPLATES_PYACE_FLASK_PATH,
    "pyace-fast": PYTHON_TEMPLATES_PYACE_FAST_PATH,
    "solution": PYTHON_TEMPLATES_SOLUTION_PATH,
}
"""A dictionary relating templates names with their paths."""



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Initialization of the frontend layout across all the steps."""


from ansys.saf.glow.client.dashclient import DashClient
from ansys_dash_treeview import AnsysDashTreeview
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, callback, callback_context, dcc, html
from dash_iconify import DashIconify

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import (
    {{ cookiecutter.__solution_definition_name }},
)
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui import problem_setup_page, monitoring_page


step_list = [
    {
        "key": "problem_setup_step",
        "text": "Problem Setup",
        "depth": 0,
    },
    {
        "key": "monitoring_step",
        "text": "Monitoring",
        "depth": 0,
    },
]


layout = html.Div(
    [
        dcc.Location(id="url", refresh=False), # represents the browser address bar and doesn't render anything
        dbc.Stack(
            [
                html.Div(
                    [
                        html.Img(
                        src = r"/assets/Graphics/ansys-solutions-horizontal-logo.png",
                        style={'width': '80%'}
                    )
                    ],
                ),
                html.Div(
                    [
                        dbc.Button(
                            "Project Name:",
                            id = "project-name",
                            disabled = True,
                            style={
                                "color": "rgba(0, 0, 0, 1)",
                                "background-color": "rgba(255, 255, 255, 1)",
                                "border-color": "rgba(0, 0, 0, 1)"
                            },
                        )
                    ],
                    className="ms-auto",
                ),
                html.Div(id="return-to-portal"),
            ],
            direction = "horizontal",
            gap = 3,
        ),
        html.Br(),
        dbc.Row(
            [
                dbc.Col(
                    AnsysDashTreeview(
                        id="navigation_tree",
                        items=step_list,
                        children=[
                            DashIconify(icon="bi:caret-right-square-fill"),
                            DashIconify(icon="bi:caret-down-square-fill"),
                        ],
                        style={"showButtons": True, "focusColor": "#ffb71b", "itemHeight": "32"},  # Ansys gold
                    ),
                    width=2,
                    style={"background-color": "rgba(242, 242, 242, 0.6)"},  # Ansys grey
                ),
                dbc.Col(
                    html.Div(
                        id="page-content",
                        style={"padding-right": "1%"}
                    ),
                    width=10
                ),
            ],
        ),
    ]
)


@callback(
    Output("return-to-portal", "children"),
    Input("url", "pathname"),
)
def return_to_portal(pathname):
    """Display Solution Portal when back-to-portal button gets selected."""

    portal_ui_url = DashClient.get_portal_ui_url()
    children = (
        []
        if portal_ui_url is None
        else [
            dbc.Button(
                "Back to Projects",
                id = "return-to-portal",
                className = "me-2",
                n_clicks = 0,
                href = portal_ui_url,
                style = {"background-color": "rgba(0, 0, 0, 1)", "border-color": "rgba(0, 0, 0, 1)"},
            )
        ]
    )
    return children


@callback(
    Output("project-name", "children"),
    Input("url", "pathname"),
)
def display_poject_name(pathname):
    """Display current project name."""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    return f"Project Name: {project.project_display_name}"


@callback(
    Output("page-content", "children"),
    [
        Input("url", "pathname"),
        Input("navigation_tree", "focus"),
    ],
    prevent_initial_call=True,
)
def display_page(pathname, value):
    """
    Display page content.

    this callback is essential for initializing the step based on the persisted
    state of the project when the browser first displays the project to the user
    given the project's URL.
    """

    project = DashClient[{{ cookiecutter.__solution_definition_name }}].get_project(pathname)
    triggered_id = callback_context.triggered[0]["prop_id"].split(".")[0]

    if triggered_id == "url":
        return problem_setup_page.layout(project.steps.problem_setup_step)
    if triggered_id == "navigation_tree":
        if value is None:
            page_layout = html.H1("Welcome!")
        elif value == "problem_setup_step":
            page_layout = problem_setup_page.layout(project.steps.problem_setup_step)
        elif value == "monitoring_step":
            page_layout = monitoring_page.layout(project.steps.monitoring_step, project.steps.problem_setup_step)
        return page_layout



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the problem setup step."""

import time
import dash_bootstrap_components as dbc

from dash.exceptions import PreventUpdate
from dash_extensions.enrich import Input, Output, State, dcc, html, callback
from ansys.saf.glow.client.dashclient import DashClient
from ansys.saf.glow.solution import MethodStatus
from ansys.solutions.dash_components.table import InputRow
from ansys.solutions.optislang.frontend_components.placeholder_table import PlaceholderTable
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{cookiecutter.__solution_definition_name}}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.problem_setup_step import ProblemSetupStep


def update_alerts(problem_setup_step: ProblemSetupStep) -> list:
    """Update all Alerts."""

    alerts = []

    # Product version alerts
    for product_name, product_data in problem_setup_step.ansys_ecosystem.items():
        alerts.append(
            html.Div(
                [
                    dbc.Button(
                        f"{product_name} Version",
                        id=f"popover-{product_name}-version-target",
                        disabled=False,
                        color=product_data["alert_color"],
                        n_clicks=0,
                    ),
                    dbc.Popover(
                        [
                            dbc.PopoverBody(product_data["alert_message"]),
                        ],
                        id=f"popover-{product_name}-version",
                        target=f"popover-{product_name}-version-target",
                        placement="top",
                        is_open=False,
                    ),
                ]
            ),
        )

    # OptiSLang solve alert
    if problem_setup_step.optislang_solve_status == "initial":
        solve_message, solve_color = "OptiSLang simulation not started.", "warning"
    elif problem_setup_step.optislang_solve_status == "stopped":
        solve_message, solve_color = "OptiSLang simulation stopped.", "warning"
    elif problem_setup_step.optislang_solve_status == "in-progress":
        solve_message, solve_color = "OptiSLang simulation in progress.", "primary"
    elif problem_setup_step.optislang_solve_status == "success":
        solve_message, solve_color = "OptiSLang simulation completed successfully.", "success"
    elif problem_setup_step.optislang_solve_status == "failure":
        solve_message, solve_color = "OptiSLang simulation failed.", "danger"

    alerts.append(
        html.Div(
            [
                dbc.Button(
                    "optiSLang Solve",
                    id="popover-optislang-solve-target",
                    disabled=False,
                    color=solve_color,
                    n_clicks=0,
                ),
                dbc.Popover(
                    [
                        dbc.PopoverBody(solve_message),
                    ],
                    id="popover-optislang-solve",
                    target="popover-optislang-solve-target",
                    placement="top",
                    is_open=False,
                ),
            ]
        ),
    )

    return alerts


def layout(problem_setup_step: ProblemSetupStep):
    """Layout of the problem setup step."""

    # Upload placeholders and assets
    if problem_setup_step.placeholder_values == {}:
        problem_setup_step.upload_project_file_to_project_directory()
        problem_setup_step.upload_properties_file_to_project_directory()
        problem_setup_step.get_default_placeholder_values()

    placeholder_table = PlaceholderTable(
        problem_setup_step.placeholder_values, problem_setup_step.placeholder_definitions
    )

    # Placeholder card for displaying parameters defined in the <project_name>.json
    placeholder_card = dbc.Accordion(
        [
            dbc.AccordionItem(
                [
                    html.Div(placeholder_table.create()),
                ],
                title="Placeholders",
                item_id="parameter-placeholders",
            )
        ]
    )

    return html.Div(
        [
            html.H1("Insert optiSLang Project Name Here", className="display-3", style={"font-size": "35px"}),
            html.P(
                "Insert a one sentence description of your app.",
                className="lead",
                style={"font-size": "20px"},
            ),
            html.Hr(className="my-2"),
            html.Br(),
            dbc.Row(
                [
                    dbc.Col(
                        [
                            dbc.Stack(
                                id="alert_messages",
                                direction="horizontal",
                                gap=3,
                            ),
                        ],
                        width=12,
                    )
                ]
            ),
            html.Br(),
            dbc.Row(
                [
                    placeholder_card,
                ],
            ),
            dbc.Row(
                [
                    dbc.Accordion(
                        [
                            dbc.AccordionItem(
                                [
                                    InputRow(
                                        "button",
                                        "check_ansys_ecosystem",
                                        "Check Ansys ecosystem",
                                        disabled=False,
                                        label_width=2,
                                        value_width=4,
                                        unit_width=1,
                                        description_width=4,
                                        class_name="button",
                                    ).get(),
                                    dcc.Loading(
                                        type="circle",
                                        fullscreen=True,
                                        color="#ffb71b",
                                        style={
                                            "background-color": "rgba(55, 58, 54, 0.1)",
                                        },
                                        children=html.Div(id="wait_ecosystem_ckeck"),
                                    ),
                                    html.Br(),
                                    InputRow(
                                        "button",
                                        "start_analysis",
                                        "Start analysis",
                                        disabled=False if problem_setup_step.ansys_ecosystem_ready else True,
                                        label_width=2,
                                        value_width=4,
                                        unit_width=1,
                                        description_width=4,
                                        class_name="button",
                                    ).get(),
                                    dcc.Loading(
                                        id="optislang_wait_spinner",
                                        type="circle",
                                        fullscreen=True,
                                        color="#ffb71b",
                                        style={
                                            "background-color": "rgba(55, 58, 54, 0.1)",
                                        },
                                        children=html.Div(id="wait_optislang_process"),
                                    ),
                                    dcc.Interval(
                                        id="solve_interval_component",
                                        interval=1 * 6000,  # in milliseconds
                                        n_intervals=0,
                                    ),
                                ],
                                title="Start Analysis",
                                item_id="start-analysis-accordion",
                            ),
                        ]
                    )
                ]
            ),
        ]
    )


for alert in ["optislang-version", "optislang-solve"]:

    @callback(
        Output(f"popover-{alert}", "is_open"),
        [Input(f"popover-{alert}-target", "n_clicks")],
        [State(f"popover-{alert}", "is_open")],
    )
    def toggle_popover(n_clicks, is_open):
        if n_clicks:
            return not is_open
        return is_open


@callback(
    Output("wait_ecosystem_ckeck", "children"),
    Output("start_analysis", "disabled"),
    Input("check_ansys_ecosystem", "n_clicks"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def check_ansys_ecosystem(n_clicks, pathname):
    """Start OptiSLang and run the simulation. Wait for the process to complete."""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step

    if n_clicks:
        problem_setup_step.check_ansys_ecosystem()
        return True, False if problem_setup_step.ansys_ecosystem_ready else True
    else:
        raise PreventUpdate


@callback(
    Output("alert_messages", "children"),
    Output("wait_optislang_process", "children"),
    Input("start_analysis", "n_clicks"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def run_optislang_synchronously(n_clicks, pathname):
    """Start OptiSLang and run the simulation. Wait for the process to complete."""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step
    if n_clicks and problem_setup_step.run_synchronously:
        problem_setup_step.run_optislang_synchronously()
        while problem_setup_step.get_long_running_method_state("run_optislang").status == MethodStatus.Running:
            time.sleep(2)
        if problem_setup_step.optislang_solve_status == "success":
            return (update_alerts(problem_setup_step), True)
        elif problem_setup_step.optislang_solve_status == "failure":
            return (update_alerts(problem_setup_step), True)
        else:
            return (update_alerts(problem_setup_step), True)
    else:
        raise PreventUpdate


@callback(
    Output("alert_messages", "children"),
    Input("start_analysis", "n_clicks"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def run_optislang_asynchronously(n_clicks, pathname):
    """Start OptiSLang and run the simulation. Wait for the process to complete."""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step
    if n_clicks and not problem_setup_step.run_synchronously:
        problem_setup_step.run_optislang_asynchronously()
        return update_alerts(problem_setup_step)
    else:
        raise PreventUpdate


@callback(
    Output("alert_messages", "children"),
    Input("solve_interval_component", "n_intervals"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def update_optislang_status(n_intervals, pathname):
    """Start OptiSLang and run the simulation. Wait for the process to complete."""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    problem_setup_step = project.steps.problem_setup_step
    if not problem_setup_step.run_synchronously and problem_setup_step.optislang_solve_status != "initial":
        problem_setup_step.get_optislsang_status()
        return update_alerts(problem_setup_step)
    else:
        return update_alerts(problem_setup_step)



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Application."""

import os
import tempfile

import dash_bootstrap_components as dbc
from dash_extensions.enrich import DashProxy, MultiplexerTransform, NoOutputTransform, TriggerTransform
import dash_uploader as du


app = DashProxy(
    __name__,
    external_stylesheets=[dbc.themes.BOOTSTRAP],
    suppress_callback_exceptions=True,
    transforms=[NoOutputTransform(), TriggerTransform(), MultiplexerTransform()],
)

# If folder doesn't exist, it will be created later
UPLOAD_DIRECTORY = os.path.join(tempfile.gettempdir(), "GLOW")
du.configure_upload(app, UPLOAD_DIRECTORY)

# !IMPORTANT Keeping the import line here to adapt with dash_uploader config, moving the import above will fail the
# dash uploader configuration
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.page import layout

app.layout = layout



"""Post-processing script for cleaning the raw rendered project."""
import os
import shutil
from pathlib import Path

import isort

from ansys.templates.utils import keep_files, remove_file

ALLOWED_BUILD_SYSTEMS = ["flit", "poetry", "setuptools"]
"""A list of all allowed build systems by the template."""

DESIRED_STRUCTURE = [
    "CHANGELOG.md",
    "CODE_OF_CONDUCT.md",
    "CONTRIBUTING.md",
    "doc/Makefile",
    "doc/make.bat",
    "doc/.vale.ini",
    "doc/styles/.gitignore",
    "doc/styles/Vocab/ANSYS/accept.txt",
    "doc/styles/Vocab/ANSYS/reject.txt",
    "doc/source/conf.py",
    "doc/source/index.rst",
    "doc/source/_static/README.md",
    "doc/source/_templates/README.md",
    "examples/README.md",
    ".flake8",
    ".github/dependabot.yml",
    ".github/labeler.yml",
    ".github/labels.yml",
    ".github/workflows/ci_cd.yml",
    ".github/workflows/label.yml",
    ".gitattributes",
    ".gitignore",
    "LICENSE",
    ".pre-commit-config.yaml",
    "pyproject.toml",
    "README.rst",
    "src/ansys/{{ cookiecutter.__product_name_slug }}/{{ cookiecutter.__library_name_slug }}/__init__.py",
    "tests/test_metadata.py",
    "tox.ini",
]
"""A list holding all desired files to be included in the project."""

def main():
    """Entry point of the script."""
    # Get baked project location path
    project_path = Path(os.getcwd())

    # Get the desired build system
    build_system = "{{ cookiecutter.build_system }}"

    # Move all requirements files into a requirements/ directory
    requirements_files = [
        f"requirements_{name}.txt" for name in ["build", "doc", "tests"]
    ]
    if build_system == "poetry":
        # Poetry required and extra dependencies are collected inside the
        # 'pyproject.toml' file. Thus, there is no need to have requirements
        # files
        for file in requirements_files:
            remove_file(file, project_path)
    else:
        os.mkdir(project_path / "requirements")
        for file in requirements_files:
            shutil.move(str(project_path / file), str(project_path / "requirements"))

    # Apply isort with desired config
    isort_config = isort.settings.Config(
        line_length="{{ cookiecutter.__max_linelength }}",
        profile="black",
    )
    filepaths_list = [
        project_path / "doc/source/conf.py",
    ]
    for filepath in filepaths_list:
        isort.api.sort_file(filepath, isort_config)

    # Remove non-desired files
    if build_system == "setuptools":
        DESIRED_STRUCTURE.append("setup.py")
    keep_files(DESIRED_STRUCTURE)


if __name__ == "__main__":
    main()



# Copyright (C) 2023 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""A collection of useful utilities and routines."""
import os
from pathlib import Path
import shutil
import tempfile

from cookiecutter.main import cookiecutter

from ansys.templates.licenses import MIT_LICENSE


def _copytree(input_path, output_path):
    """
    Recursively copy all the contents of a directory into desired one.

    Parameters
    ----------
    input_path : ~pathlib.Path
        Path of the source directory to be copied.
    output_path : ~pathlib.Path
        Path of the destination directory.

    """
    # Create output directory if it does not exist and ensure permission bits
    if not os.path.exists(output_path):
        os.makedirs(output_path)
        shutil.copystat(input_path, output_path)

    # Get all file directories from the input directory
    files_list = os.listdir(input_path)

    for file in files_list:
        # Create source and destination file paths
        source_path = os.path.join(input_path, file)
        dest_path = os.path.join(output_path, file)

        # Recursion is used in case a directory is found
        if os.path.isdir(source_path):
            _copytree(source_path, dest_path)
        else:
            shutil.copy2(source_path, dest_path)


def _copy_common_template_files(common_path, project_path):
    """
    Copy common template files into desired project directory.

    Parameters
    ----------
    common_path : ~pathlib.Path
        Path to the common template directory.
    project_path : ~pathlib.Path
        Path to the baked project directory.

    """
    _copytree(
        common_path / "{{cookiecutter.__project_name_slug}}",
        project_path / "{{cookiecutter.__project_name_slug}}",
    )


def _copy_all_template_files(template_path, project_path):
    """
    Copy all template files including cookiecutter.json and hooks/ directory.

    Parameters
    ----------
    template_path : ~pathlib.Path
        Path to the template directory.
    project_path : ~pathlib.Path
        Path to the baked project directory.

    """
    _copytree(
        template_path,
        project_path,
    )


def _include_license(license_path, project_path):
    """
    Include a desired license into the baked project.

    Parameters
    ----------
    license_path : ~pathlib.Path
        Path to the license template.
    project_path : ~pathlib.Path
        Path to the baked project directory.

    Notes
    -----
    This function is intended to be used during the pre_gen_project.py hook.

    """
    shutil.copyfile(license_path, project_path / "{{cookiecutter.__project_name_slug}}/LICENSE")


def remove_file(filename, project_path=Path(os.getcwd())):
    """Remove desired file being given its relative path to project.

    Parameters
    ----------
    filename : str
        Filename relative to project root directory.
    project_path : Path
        Project's root directory.

    """
    filepath = project_path / filename
    filepath.unlink()


def keep_files(files_list, project_path=Path(os.getcwd())):
    """Remove undesired files except given ones from project.

    Parameters
    ----------
    files_list : list
        Desired file names relative to project's root directory.
    project_path : Path
        Project's root directory.

    """
    # Collect all the files in the project
    all_project_files = project_path.glob("**/*")

    # Fix path name according to OS flavor
    separator, new_separator = ("/", "\\") if os.name != "posix" else ("/", "/")
    desired_files = [file.replace(separator, new_separator) for file in files_list]

    folders = []
    # Removing undesired files
    for file in all_project_files:
        if str(file.relative_to(project_path)).replace(separator, new_separator) not in desired_files:
            file.unlink() if file.is_file() else folders.append(file)

    folders = sorted(folders, reverse=True)

    # Removing undesired empty folders
    [folder.rmdir() for folder in folders if not os.listdir(str(folder))]


def bake_template(template_path, output_path, license_path=MIT_LICENSE, **cookiecutter_kwargs):
    """
    Bakes project using desired template and common files.

    Parameters
    ----------
    template_path: ~pathlib.Path
        Path to the template.
    output_path: ~pathlib.Path
        Output path for the baked template.
    license_path: ~pathlib.Path
        Path to license file. Default is MIT.
    **cookiecutter_kwargs: dict
        Additional cookiecutter keyword arguments.

    Notes
    -----
    Files from the common directory need to be copied before the cookiecutter
    context initializes. Otherwise, copied files by a hook will not be
    rendered. This function creates a temporary directory where the common and
    desired template are combined so then cookiecutter can be executed.

    """
    # Create a temporary directory to be used as the final template source
    with tempfile.TemporaryDirectory() as tmp_template_path:

        # The common directory can be obtained from the template path
        common_path = template_path / "../common"

        # Copy the common and desired template files
        _copy_common_template_files(common_path, Path(str(tmp_template_path)))
        _copy_all_template_files(template_path, Path(str(tmp_template_path)))

        # Copy license file
        _include_license(license_path, Path(str(tmp_template_path)))

        # Bake the temporary project using cookiecutter with desired options
        cookiecutter(str(tmp_template_path), output_dir=str(output_path), **cookiecutter_kwargs)



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Backend of the monitoring step."""

from ansys.saf.glow.solution import StepModel, StepSpec, transaction
from ansys.solutions.optislang.wrapper.monitoring import Monitoring


class MonitoringStep(StepModel):
    """Step model of the monitoring step."""

    # Frontend persistence
    available_root_nodes: list = []
    available_system_nodes: dict = {}
    available_subsystem_nodes: dict = {}
    available_fields: list = []
    selected_root_node: str = None
    selected_system_node: str = None
    selected_subsystem_node: str = None
    selected_node_kind: str = None
    selected_design_id: str = None
    selected_axis_1: str = None
    selected_axis_2: str = None
    selected_axis_3: str = None
    summary: dict = None
    design_table: dict = None
    parameter_ranges: dict = None
    response_ranges: dict = None
    constraint_ranges: dict = None
    objective_ranges: dict = None
    anthill: dict = None
    history: dict = None
    parallel_coordinates: dict = None
    status_overview: dict = None
    project_state: dict = None

    @transaction(
        self=StepSpec(upload=["available_root_nodes", "available_system_nodes", "available_subsystem_nodes"]),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_system_hierarchy(self, problem_setup_step):
        """Update root nodes, system nodes and subsystem nodes."""

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        self.available_root_nodes = monitoring.get_root_node_names()
        for root_node_name in self.available_root_nodes:
            self.available_system_nodes[root_node_name] = monitoring.get_root_node_children(root_node_name)
            for system_node_name in self.available_system_nodes[root_node_name]:
                self.available_subsystem_nodes[system_node_name] = monitoring.get_system_node_children(
                    root_node_name, system_node_name
                )

        monitoring.dispose()

    @transaction(
        self=StepSpec(
            download=["selected_root_node", "selected_system_node", "selected_subsystem_node"],
            upload=["selected_node_kind"],
        ),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_node_kind(self, problem_setup_step):
        """Update root nodes, system nodes and subsystem nodes."""

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        if self.selected_subsystem_node:
            actor_name = self.selected_subsystem_node
        elif self.selected_system_node:
            actor_name = self.selected_system_node
        elif self.selected_root_node:
            actor_name = self.selected_root_node
        else:
            raise Exception("No selection.")

        self.selected_node_kind = monitoring._get_actor_type(actor_name)

        monitoring.dispose()

    @transaction(
        self=StepSpec(download=["selected_root_node"], upload=["summary"]),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_project_summary(self, problem_setup_step):
        """Get the root summary."""

        if self.selected_root_node is None:
            self.selected_root_node = self.available_root_nodes[0]

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        self.summary = monitoring.get_project_summary(self.selected_root_node)

        monitoring.dispose()

    @transaction(
        self=StepSpec(download=["selected_system_node", "selected_subsystem_node"], upload=["summary"]),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_summary(self, problem_setup_step):
        """Get the design table of a system."""

        if self.selected_subsystem_node:
            actor_name = self.selected_subsystem_node
        elif self.selected_system_node:
            actor_name = self.selected_system_node
        else:
            raise Exception("No actor selected.")

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        self.summary = monitoring.get_summary(actor_name)

        monitoring.dispose()

    @transaction(
        self=StepSpec(
            download=["selected_root_node", "selected_system_node", "selected_subsystem_node"], upload=["design_table"]
        ),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_design_table(self, problem_setup_step):
        """Get the design table of a system."""

        is_root = False
        if self.selected_subsystem_node:
            actor_name = self.selected_subsystem_node
        elif self.selected_system_node:
            actor_name = self.selected_system_node
        elif self.selected_root_node:
            is_root = True
            actor_name = self.selected_root_node
        else:
            raise Exception("No actor selected.")

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        if is_root:
            self.design_table = monitoring.get_root_design_table(actor_name, to_dataframe=False)
        else:
            self.design_table = monitoring.get_design_table(actor_name, to_dataframe=False)

        monitoring.dispose()

    @transaction(
        self=StepSpec(
            download=[
                "selected_system_node",
                # "selected_design_id",
                # "selected_axis_1",
                # "selected_axis_2",
                # "selected_axis_3",
            ],
            upload=[
                "selected_design_id",
                "selected_axis_1",
                "selected_axis_2",
                "selected_axis_3",
                "design_table",
                "parameter_ranges",
                "response_ranges",
                "constraint_ranges",
                "objective_ranges",
                "anthill",
                "history",
                "parallel_coordinates",
            ],
        ),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_visualization_data(self, problem_setup_step):
        """Get visualization data."""

        if self.selected_system_node:
            actor_name = self.selected_system_node
        else:
            raise Exception("No actor selected.")

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        self.design_table = monitoring.get_design_table(actor_name, to_dataframe=False)

        if not self.selected_design_id:
            self.selected_design_id = self.design_table["Design"][0]

        self.parameter_ranges = monitoring.get_field_ranges(
            actor_name, "parameter", selected_design_id=self.selected_design_id
        )
        self.response_ranges = monitoring.get_field_ranges(
            actor_name, "response", selected_design_id=self.selected_design_id
        )
        self.constraint_ranges = monitoring.get_field_ranges(
            actor_name, "constraint", selected_design_id=self.selected_design_id
        )
        self.objective_ranges = monitoring.get_field_ranges(
            actor_name, "objective", selected_design_id=self.selected_design_id
        )

        if not self.selected_axis_1:
            self.selected_axis_1 = list(self.parameter_ranges.keys())[0]
        if not self.selected_axis_2:
            if len(self.parameter_ranges.keys()) >= 2:
                self.selected_axis_2 = list(self.parameter_ranges.keys())[1]
            else:
                self.selected_axis_2 = self.selected_axis_1
        if not self.selected_axis_3:
            if len(self.parameter_ranges.keys()) >= 3:
                self.selected_axis_3 = list(self.parameter_ranges.keys())[2]
            else:
                self.selected_axis_3 = self.selected_axis_1

        self.anthill = monitoring.get_anthill(
            actor_name,
            selected_design_id=self.selected_design_id,
            axis_1=self.selected_axis_1,
            axis_2=self.selected_axis_2,
            to_dataframe=False,
        )
        self.history = monitoring.get_history(
            actor_name, selected_design_id=self.selected_design_id, axis_1=self.selected_axis_1, to_dataframe=False
        )
        self.parallel_coordinates = monitoring.get_parallel_coordinates(actor_name)

        monitoring.dispose()

    @transaction(
        self=StepSpec(
            download=["selected_root_node", "selected_system_node", "selected_subsystem_node"],
            upload=["status_overview"],
        ),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_status_overview(self, problem_setup_step):
        """Get the design table of a system."""

        if self.selected_subsystem_node:
            actor_name = self.selected_subsystem_node
        elif self.selected_system_node:
            actor_name = self.selected_system_node
        elif self.selected_root_node:
            actor_name = self.selected_root_node
        else:
            raise Exception("No actor selected.")

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()

        self.status_overview = monitoring.get_status_overview(actor_name, to_dataframe=False)

        monitoring.dispose()

    @transaction(
        self=StepSpec(upload=["project_state"]),
        problem_setup_step=StepSpec(download=["project_file", "host", "port"]),
    )
    def get_project_state(self, problem_setup_step):
        """Get project state."""

        monitoring = Monitoring(
            project_file=problem_setup_step.project_file.path,
            host=problem_setup_step.host,
            port=problem_setup_step.port,
        )

        monitoring.initialize()
        self.project_state = monitoring.get_scenery_and_node_status_view_data()
        monitoring.dispose()


# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""

import dash_bootstrap_components as dbc
from dash_extensions.enrich import dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep


def layout(monitoring_step: MonitoringStep):
    """Layout of the second step UI."""

    return html.Div(
        [
            html.Br(),
            dbc.Row(
                [
                    dcc.Markdown("**Result files: Under construction**", style={"font-size": "15px"}),
                ],
            ),
        ]
    )



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Backend of the problem setup step."""

import platform
from pathlib import Path
import time

from ansys.saf.glow.solution import StepModel, StepSpec, FileReference, transaction, long_running
from ansys.solutions.optislang.parser.project_properties import ProjectProperties
from ansys.solutions.optislang.wrapper.monitoring import Monitoring
from ansys.solutions.optislang.wrapper.orchestrator import OptiSLangOrchestrator
from ansys.solutions.products_ecosystem.controller import AnsysProductsEcosystemController
from ansys.solutions.products_ecosystem.utils import convert_to_long_version, convert_to_short_version
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.model.utils import check_if_file_is_empty


class ProblemSetupStep(StepModel):
    """Step model of the problem setup step."""

    # Frontend persistence
    ansys_ecosystem_ready: bool = False
    optislang_solve_status: str = "initial"  # initial, in-progress, failure, success
    placeholder_values: dict = {}
    placeholder_definitions: dict = {}

    # Backend data model
    host: str = "127.0.0.1"
    port: int = 5310  # 53538
    run_synchronously: bool = False
    ansys_ecosystem: dict = {
        "optiSLang": {
            "authorized_versions": ["2022.2", "2023.1"],
            "installed_versions": [],
            "compatible_versions": [],
            "selected_version": None,
            "alert_message": "optiSLang install not checked.",
            "alert_color": "warning",
        }
    }

    # File storage
    project_file: FileReference = FileReference("Problem_Setup/{{ cookiecutter.__optiSLang_project_file_name }}")
    properties_file: FileReference = FileReference("Problem_Setup/{{ cookiecutter.__optiSLang_properties_file_name }}")

    @transaction(self=StepSpec(download=["properties_file"], upload=["placeholder_values", "placeholder_definitions"]))
    def get_default_placeholder_values(self):
        """Get placeholder values and definitions using the ProjectProperties class."""

        pp = ProjectProperties()
        pp.read_file(self.properties_file.path)
        placeholders = pp.get_properties()['placeholders']
        self.placeholder_values = placeholders.get('placeholder_values')
        self.placeholder_definitions = placeholders.get('placeholder_definitions')

    @transaction(self=StepSpec(upload=["project_file"]))
    def upload_project_file_to_project_directory(self) -> None:
        """Upload optiSLang project file to project directory."""

        original_project_file = (
            Path(__file__).parent.absolute().parent / "model" / "assets" / "{{ cookiecutter.__optiSLang_project_file_name }}"
        )
        self.project_file.write_bytes(original_project_file.read_bytes())

    @transaction(self=StepSpec(upload=["properties_file"]))
    def upload_properties_file_to_project_directory(self) -> None:
        """Upload optiSLang properties to project directory."""

        original_properties_file = (
            Path(__file__).parent.absolute().parent / "model" / "assets" / "{{ cookiecutter.__optiSLang_properties_file_name }}"
        )
        self.properties_file.write_bytes(original_properties_file.read_bytes())

    @transaction(
        self=StepSpec(
            upload=["ansys_ecosystem", "ansys_ecosystem_ready"],
        )
    )
    def check_ansys_ecosystem(self) -> None:
        """Check if Ansys Products are installed and if the appropriate versions are available."""

        self.ansys_ecosystem_ready = True

        controller = AnsysProductsEcosystemController()

        for product_name in self.ansys_ecosystem.keys():

            self.ansys_ecosystem[product_name]["installed_versions"] = controller.get_installed_versions(
                product_name.lower(), outout_format="long"
            )
            self.ansys_ecosystem[product_name]["compatible_versions"] = [
                product_version
                for product_version in self.ansys_ecosystem[product_name]["installed_versions"]
                if product_version in self.ansys_ecosystem[product_name]["authorized_versions"]
            ]

            if len(self.ansys_ecosystem[product_name]["installed_versions"]) == 0:
                alert_message = f"No installation of {product_name} found in the machine {platform.node()}."
                alert_color = "danger"
                self.ansys_ecosystem_ready = False
            elif len(self.ansys_ecosystem[product_name]["compatible_versions"]) == 0:
                alert_message = (
                    f"None of the authorized versions of {product_name} "
                    f"is installed in the machine {platform.node()}.\n"
                )
                alert_message += "At least one of these versions is required:"
                for authorized_version in self.ansys_ecosystem[product_name]["authorized_versions"]:
                    self.ansys_ecosystem[product_name][
                        "alert_message"
                    ] += f" {convert_to_long_version(authorized_version)}"
                alert_message += "."
                alert_color = "danger"
                self.ansys_ecosystem_ready = False
            else:
                self.ansys_ecosystem[product_name]["selected_version"] = self.ansys_ecosystem[product_name][
                    "compatible_versions"
                ][
                    -1
                ]  # Latest
                alert_message = f"{product_name} install detected. Compatible versions are:"
                for compatible_version in self.ansys_ecosystem[product_name]["compatible_versions"]:
                    alert_message += f" {convert_to_long_version(compatible_version)}"
                alert_message += ".\n"
                alert_message += "Selected version is %s." % (self.ansys_ecosystem[product_name]["selected_version"])
                alert_color = "success"
            self.ansys_ecosystem[product_name]["alert_message"] = alert_message
            self.ansys_ecosystem[product_name]["alert_color"] = alert_color

    @transaction(
        self=StepSpec(
            download=["ansys_ecosystem", "project_file", "properties_file", "host", "port"],
            upload=["optislang_solve_status"],
        )
    )
    @long_running
    def run_optislang_synchronously(self) -> None:
        """Start optiSLang and run the project."""

        stderr_file = str(self.project_file.project_path / "Problem_Setup" / "stderr.log")

        # Start optiSLang
        osl = OptiSLangOrchestrator(
            project_file=self.project_file.path,
            properties_file=self.properties_file.path,
            output_file="hook_optimization_output.json",
            version=convert_to_short_version(self.ansys_ecosystem["optiSLang"]["selected_version"]),
            host=self.host,
            port=self.port,
        )
        osl.start()

        # Wait for optiSLang to complete
        while True:
            # Check the status of the optiSLang solve
            if osl.get_status() == "processing":
                self.optislang_solve_status = "in-progress"
            elif osl.get_status() == "succeeded":
                self.optislang_solve_status = "success"
            elif osl.get_status() == "failed":
                self.optislang_solve_status = "failure"
            elif (
                osl.get_status() == "stopped"
            ):  # Case where optiSLang stops without error, to further continue the solve.
                if not check_if_file_is_empty(stderr_file):
                    self.optislang_solve_status = "failure"
            else:
                osl.stop()
                raise Exception(f"ERROR: Unknown status: {osl.get_status()}.")
            # Exit
            if osl.get_status() == "succeeded" or self.optislang_solve_status == "failure":
                osl.dispose()
                # osl.stop()
                break
            time.sleep(1)

        osl.dispose()

    @transaction(
        self=StepSpec(
            download=["ansys_ecosystem", "project_file", "properties_file", "host", "port"],
            upload=[
                "optislang_solve_status",
            ],
        )
    )
    @long_running
    def run_optislang_asynchronously(self) -> None:
        """Start optiSLang and run the project."""

        stderr_file = str(self.project_file.project_path / "Problem_Setup" / "stderr.log")

        # Start optiSLang
        osl = OptiSLangOrchestrator(
            project_file=self.project_file.path,
            properties_file=self.properties_file.path,
            output_file="hook_optimization_output.json",
            version=convert_to_short_version(self.ansys_ecosystem["optiSLang"]["selected_version"]),
            host=self.host,
            port=self.port,
        )
        osl.start()

        # Wait for optiSLang to complete
        if osl.get_status() == "processing":
            self.optislang_solve_status = "in-progress"
        elif osl.get_status() == "succeeded":
            self.optislang_solve_status = "success"
        elif osl.get_status() == "failed":
            self.optislang_solve_status = "failure"
        elif osl.get_status() == "stopped":  # Case where optiSLang stops without error, to further continue the solve.
            if not check_if_file_is_empty(stderr_file):
                self.optislang_solve_status = "failure"
            else:
                self.optislang_solve_status = "stopped"

        osl.dispose()

    @transaction(
        self=StepSpec(
            download=["project_file", "host", "port"],
            upload=[
                "optislang_solve_status",
            ],
        )
    )
    def get_optislsang_status(self) -> None:
        """Start optiSLang and run the project."""

        monitoring = Monitoring(project_file=self.project_file.path, host=self.host, port=self.port)

        monitoring.initialize()

        monitoring.get_state()

        if monitoring.get_state().lower() == "processing":
            self.optislang_solve_status = "in-progress"
        elif monitoring.get_state().lower() == "finished":
            self.optislang_solve_status = "success"
        elif monitoring.get_state().lower() == "failed":
            self.optislang_solve_status = "failure"
        elif (
            monitoring.get_state().lower() == "stopped"
        ):  # Case where optiSLang stops without error, to further continue the solve.
            self.optislang_solve_status = "stopped"
        else:
            self.optislang_solve_status = "failure"

        monitoring.dispose()



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the first step."""


from ansys.saf.glow.client.dashclient import DashClient
from dash_extensions.enrich import Input, Output, State, callback, dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{cookiecutter.__solution_definition_name}}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.first_step import FirstStep


def layout(step: FirstStep):
    """Layout of the first step UI."""
    return html.Div(
        [
            dcc.Markdown("""#### First step""", className="display-3"),
            dcc.Markdown("""###### Compute the sum of two numbers.""", className="display-3"),
            html.Hr(className="my-2"),
            html.Br(),
            html.Div(["First Argument: ", dcc.Input(id="first-arg", value=step.first_arg, type="number")]),
            html.Div(["Second Argument: ", dcc.Input(id="second-arg", value=step.second_arg, type="number")]),
            html.Button("Calculate", id="calculate", n_clicks=0),
            html.P(id="result", children=f"Result:{step.result}"),
        ]
    )


# This is an example callback which stores entered data and executes a method.
# This is the only place where user entered data is persisted in this Dash app.
# Notice that the project is referenced by a call to get_project and
# the url is a State argument to the callback
@callback(
    Output("result", "children"),
    Input("calculate", "n_clicks"),
    State("first-arg", "value"),
    State("second-arg", "value"),
    State("url", "pathname"),
    prevent_initial_call=True,
)
def calculate(n_clicks, first_arg, second_arg, pathname):
    """Callback function to trigger the computation."""
    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    step = project.steps.first_step
    step.first_arg = first_arg
    step.second_arg = second_arg
    step.calculate()
    return f"Result:{step.result}"



{%- if cookiecutter.copyright != "None" -%}
# Copyright (c) {% now "utc", '%Y' %}, {{ cookiecutter.copyright }}. Unauthorised use, distribution or duplication is prohibited
{% endif %}

"""
{{ cookiecutter.project_name }}.

{{ cookiecutter.library_name }}
"""

from datetime import datetime

from logger import Logger

logger = Logger.init("{{ cookiecutter.project_name }}.{{ cookiecutter.library_name }}")


def get_date_and_time():
    """Compute today's datetime."""
    return datetime.today().strftime("%Y-%m-%d-%H:%M:%S")


if __name__ == "__main__":
    logger.info(f"Hello! Welcome, we are {get_date_and_time()}")



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""General purpose functions."""

from collections import defaultdict
import os


def check_if_exists_in_file(string: str, file: str) -> bool:
    """Check if a string exists in a file."""

    with open(file) as f:
        if string in f.read():
            return True
    return False


def check_if_file_is_empty(file: str) -> bool:
    """Check file size. Return True if empty file, else False."""

    return os.stat(file).st_size == 0


def get_duplicates_from_list(sequence: list) -> list:
    """Return the duplicates items in a list.

    Reference: https://stackoverflow.com/questions/5419204/index-of-duplicates-items-in-a-python-list
    """

    tally = defaultdict(list)
    for i, item in enumerate(sequence):
        tally[item].append(i)
    duplicates = dict([(key, locs) for key, locs in tally.items() if len(locs) > 1])
    return list(duplicates.keys())



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the scenery view tab."""

import dash_bootstrap_components as dbc
from dash_extensions.enrich import html
import optislang_dash_lib

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep


def layout(monitoring_step: MonitoringStep):
    """Layout of the scenery view tab."""

    monitoring_step.get_project_state() # fetches data on page load

    return html.Div(
        [
            html.Br(),
            dbc.Row(
                [
                    html.Div(
                        [
                            optislang_dash_lib.SceneryComponent(
                                id="input",
                                project_state=monitoring_step.project_state,
                            ),
                        ]
                    )
                ],
            ),
        ]
    )



"""Post-processing script for cleaning the raw rendered project."""
import os
from pathlib import Path

from ansys.templates.utils import keep_files

ALLOWED_BUILD_SYSTEMS = ["flit"]
"""A list of all allowed build systems by the template."""

DESIRED_STRUCTURE = [
    "CHANGELOG.md",
    "CODE_OF_CONDUCT.md",
    "CONTRIBUTING.md",
    ".flake8",
    ".github/workflows/build_and_test_library.yml",
    ".github/workflows/generate_library.yml",
    ".github/dependabot.yml",
    ".gitattributes",
    ".gitignore",
    "LICENSE",
    ".pre-commit-config.yaml",
    ".m2/settings.xml",
    "yaml/{{ cookiecutter.yaml_file_name }}",
    "pom.xml"
]
"""A list holding all desired files to be included in the project."""


def main():
    """Entry point of the script."""

    # Get baked project location path
    project_path = Path(os.getcwd())

    # Get the desired build system
    build_system = "{{ cookiecutter.build_system }}"

    keep_files(DESIRED_STRUCTURE)


if __name__ == "__main__":
    main()



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""

from ansys.solutions.dash_components.table import OutputRow
import dash_bootstrap_components as dbc
from dash_extensions.enrich import dcc, html

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep


def layout(monitoring_step: MonitoringStep):
    """Layout of the second step UI."""

    monitoring_step.get_project_summary()

    font_size = "15px"

    return html.Div(
        [
            html.Br(),
            dbc.Row(
                [
                    dcc.Markdown("**Project Information**", style={"font-size": "20px"}),
                    OutputRow(
                        "label",
                        "project-state",
                        "State",
                        row_default_value=monitoring_step.summary["state"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "project-id",
                        "Id",
                        row_default_value=monitoring_step.summary["project_id"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "project-name",
                        "Name",
                        row_default_value=monitoring_step.summary["name"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "project-machine",
                        "Machine",
                        row_default_value=monitoring_step.summary["machine"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "project-location",
                        "Location",
                        row_default_value=monitoring_step.summary["location"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "project-directory",
                        "Project directory",
                        row_default_value=monitoring_step.summary["working_dir"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "project-owner",
                        "Owner",
                        row_default_value=monitoring_step.summary["user"],
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "registered",
                        "Registered",
                        row_default_value=None,
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                    OutputRow(
                        "label",
                        "lock-info",
                        "Lock info",
                        row_default_value=None,
                        row_description="",
                        label_width=2,
                        value_width=10,
                        unit_width=0,
                        description_width=0,
                        font_size=font_size,
                        text_align="left",
                    ).get(),
                ],
            ),
        ]
    )



"""
{{ cookiecutter.__pkg_name | replace('-', ' ') | title }}.
"""

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__.replace(".", "-"))



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""


from ansys.saf.glow.client.dashclient import DashClient
import dash_bootstrap_components as dbc
from dash_extensions.enrich import Input, Output, State, callback, dcc, html
import pandas as pd

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.definition import {{cookiecutter.__solution_definition_name}}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.colorscale import ANSYS_COLORSCALE
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui.monitoring_tabs.design_table_page import DesignTable


def layout(monitoring_step: MonitoringStep):
    """Layout of the visualization tab UI."""

    monitoring_step.get_visualization_data()

    axes_selection = dbc.CardBody(
        dbc.Card(
            [
                dcc.Markdown("""Axes"""),
                html.Div(
                    [
                        dcc.Markdown("""1st"""),
                        dcc.Dropdown(
                            [
                                key
                                for key in monitoring_step.design_table.keys()
                                if key not in ["Design", "Feasible", "Status", "Pareto"]
                            ],
                            monitoring_step.selected_axis_1,
                            id="axis_1",
                            multi=False,
                            clearable=True,
                            disabled=False,
                        ),
                    ],
                    style={"width": "100%", "display": "inline-block"},
                ),
                html.Div(
                    [
                        dcc.Markdown("""2nd"""),
                        dcc.Dropdown(
                            [
                                key
                                for key in monitoring_step.design_table.keys()
                                if key not in ["Design", "Feasible", "Status", "Pareto"]
                            ],
                            monitoring_step.selected_axis_2,
                            id="axis_2",
                            multi=False,
                            clearable=True,
                            disabled=False,
                        ),
                    ],
                    style={"width": "100%", "display": "inline-block"},
                ),
                html.Div(
                    [
                        dcc.Markdown("""3rd"""),
                        dcc.Dropdown(
                            [
                                key
                                for key in monitoring_step.design_table.keys()
                                if key not in ["Design", "Feasible", "Status", "Pareto"]
                            ],
                            monitoring_step.selected_axis_3,
                            id="axis_3",
                            multi=False,
                            clearable=True,
                            disabled=False,
                        ),
                    ],
                    style={"width": "100%", "display": "inline-block"},
                ),
            ]
        )
    )

    design_selection = dbc.CardBody(
        dbc.Card(
            [
                dcc.Markdown("""Designs"""),
                html.Div(
                    [
                        dcc.Markdown("""Selection"""),
                        dcc.Dropdown(
                            monitoring_step.design_table["Design"],
                            monitoring_step.selected_design_id,
                            id="design_id",
                            multi=False,
                            clearable=True,
                            disabled=False,
                        ),
                    ],
                    style={"width": "100%", "display": "inline-block"},
                ),
            ]
        )
    )

    parameter_ranges_graph = dcc.Loading(
        id="loading_parameter_ranges_graph",
        type="circle",
        color="#ffb71b",
        children=[dcc.Graph(id="parameter_ranges_graph")],
    )

    response_ranges_graph = dcc.Loading(
        id="loading_response_ranges_graph",
        type="circle",
        color="#ffb71b",
        children=[dcc.Graph(id="response_ranges_graph")],
    )

    criteria_ranges_graph = dcc.Loading(
        id="loading_criteria_ranges_graph",
        type="circle",
        color="#ffb71b",
        children=[dcc.Graph(id="criteria_ranges_graph")],
    )

    anthill_graph = (
        dcc.Loading(
            id="loading_anthill_graph",
            type="circle",
            color="#ffb71b",
            children=[dcc.Graph(id="anthill_graph")],
        ),
    )

    history_graph = dcc.Loading(
        id="loading_history_graph",
        type="circle",
        color="#ffb71b",
        children=[dcc.Graph(id="history_graph")],
    )

    parallel_coordinates_graph = dcc.Loading(
        id="load_parallel_coordinates_graph",
        type="circle",
        color="#ffb71b",
        children=[dcc.Graph(id="parallel_coordinates_graph")],
    )

    design_table = DesignTable(pd.DataFrame(monitoring_step.design_table))

    return html.Div(
        [
            html.Br(),
            dbc.Row(
                [
                    dbc.Col([dbc.Row(axes_selection), dbc.Row(design_selection)], width=1),
                    dbc.Col(
                        [
                            dbc.Row(
                                [
                                    dbc.Col(parameter_ranges_graph, md=4),
                                    dbc.Col(response_ranges_graph, md=4),
                                    dbc.Col(criteria_ranges_graph, md=4),
                                ],
                                className="g-0",
                            ),
                            dbc.Row(
                                [
                                    dbc.Col(anthill_graph, md=4),
                                    dbc.Col(history_graph, md=4),
                                    dbc.Col(parallel_coordinates_graph, md=4),
                                ],
                                className="g-0",
                            ),
                            dbc.Row([dbc.Col(design_table.render(), md=12)]),
                        ],
                        width={"size": 9, "offset": 0, "order": "2"},
                    ),
                ]
            ),
        ]
    )


@callback(
    Output("parameter_ranges_graph", "figure"),
    Output("response_ranges_graph", "figure"),
    Output("criteria_ranges_graph", "figure"),
    Output("anthill_graph", "figure"),
    Output("history_graph", "figure"),
    Output("parallel_coordinates_graph", "figure"),
    Input("axis_1", "value"),
    Input("axis_2", "value"),
    Input("axis_3", "value"),
    Input("design_id", "value"),
    State("url", "pathname"),
)
def update_graphs(selected_axis_1, selected_axis_2, selected_axis_3, selected_design_id, pathname):
    """"""

    project = DashClient[{{cookiecutter.__solution_definition_name}}].get_project(pathname)
    monitoring_step = project.steps.monitoring_step

    monitoring_step.selected_design_id = selected_design_id
    monitoring_step.selected_axis_1 = selected_axis_1
    monitoring_step.selected_axis_2 = selected_axis_2
    monitoring_step.selected_axis_3 = selected_axis_3

    monitoring_step.get_visualization_data()

    return (
        {
            "data": [
                {
                    "x": list(monitoring_step.parameter_ranges.keys()),
                    "y": [parameter["range"] for parameter in monitoring_step.parameter_ranges.values()],
                    "type": "bar",
                    "name": "parameters",
                    "marker": {
                        "color": [parameter["range"] for parameter in monitoring_step.parameter_ranges.values()],
                        "colorscale": ANSYS_COLORSCALE,
                    },
                },
            ],
            "layout": {"title": "Parameter Ranges", "yaxis": {"title": "Relative to range", "range": [0, 100]}},
        },
        {
            "data": [
                {
                    "x": list(monitoring_step.response_ranges.keys()),
                    "y": [response["range"] for response in monitoring_step.response_ranges.values()],
                    "type": "bar",
                    "name": "responses",
                    "marker": {
                        "color": [response["range"] for response in monitoring_step.response_ranges.values()],
                        "colorscale": ANSYS_COLORSCALE,
                    },
                },
            ],
            "layout": {"title": "Response Ranges", "yaxis": {"title": "Relative to range", "range": [0, 100]}},
        },
        {
            "data": [
                {
                    "x": list(monitoring_step.objective_ranges.keys()),
                    "y": [objective["range"] for objective in monitoring_step.objective_ranges.values()],
                    "type": "bar",
                    "name": "objective",
                    "marker": {
                        "color": [objective["range"] for objective in monitoring_step.objective_ranges.values()],
                        "colorscale": ANSYS_COLORSCALE,
                    },
                },
                {
                    "x": list(monitoring_step.constraint_ranges.keys()),
                    "y": [constraint["range"] for constraint in monitoring_step.constraint_ranges.values()],
                    "type": "bar",
                    "name": "constraints",
                    "marker": {
                        "color": [constraint["range"] for constraint in monitoring_step.constraint_ranges.values()],
                        "colorscale": ANSYS_COLORSCALE,
                    },
                },
            ],
            "layout": {"title": "Criteria Ranges", "yaxis": {"title": "Relative to range", "range": [0, 100]}},
        },
        {
            "data": [
                {
                    "x": list(monitoring_step.anthill[monitoring_step.selected_axis_1]),
                    "y": list(monitoring_step.anthill[monitoring_step.selected_axis_2]),
                    "type": "scatter",
                    "name": "anthill",
                    "mode": "markers",
                },
            ],
            "layout": {
                "title": "Anthill",
                "xaxis": {"title": monitoring_step.selected_axis_1},
                "yaxis": {"title": monitoring_step.selected_axis_2},
            },
        },
        {
            "data": [
                {
                    "x": [int(design.split(".")[-1]) for design in list(monitoring_step.history["Design"])],
                    "y": list(monitoring_step.history[monitoring_step.selected_axis_1]),
                    "type": "scatter",
                    "name": "history",
                    "mode": "markers+lines",
                },
            ],
            "layout": {
                "title": "History",
                "xaxis": {"title": "Designs"},
                "yaxis": {"title": monitoring_step.selected_axis_1},
            },
        },
        {
            "data": [
                {
                    "dimensions": [
                        {
                            "range": monitoring_step.parallel_coordinates[field_name]["range"],
                            "label": field_name,
                            "values": monitoring_step.parallel_coordinates[field_name]["values"],
                        }
                        for field_name in monitoring_step.parallel_coordinates.keys()
                    ],
                    "type": "parcoords",
                    "line_color": "rgb(0, 0, 0)",
                },
            ],
            "layout": {
                "title": "Parallel Coordinates",
            },
        },
    )



# ©2022, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Sphinx documentation configuration file."""

# ==================================================== [Imports] ==================================================== #

from datetime import datetime
import os
from pathlib import Path

from ansys_sphinx_theme import ansys_favicon, get_version_match
import toml

# ============================================== [Project Information] ============================================== #

package_configuration = toml.load(Path(__file__).parent.parent.parent.absolute() / "pyproject.toml")

# Project information
project = package_configuration["tool"]["poetry"]["name"]
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
release = version = package_configuration["tool"]["poetry"]["version"]
cname = os.getenv("DOCUMENTATION_CNAME", "nocname.com")

# ============================================ [Options for HTML output] ============================================ #

# Select desired logo, theme, and declare the html title
html_logo = str(Path(__file__).parent.absolute() / "_static" / "ansys-solutions-logo-black-background.png")
html_theme = "ansys_sphinx_theme"
html_favicon = ansys_favicon
html_short_title = html_title = package_configuration["tool"]["poetry"]["name"]

# specify the location of your github repo
html_theme_options = {
    "github_url": package_configuration["tool"]["poetry"]["repository"],
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "additional_breadcrumbs": [
        ("PyAnsys", "https://docs.pyansys.com/"),
    ],
    "switcher": {
        "json_url": f"{cname}/release/versions.json",
        "version_match": get_version_match(version),
    },
    "navbar_end": ["version-switcher", "theme-switcher", "navbar-icon-links"],
}

# Sphinx extensions
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "numpydoc",
    "sphinx.ext.intersphinx",
    "sphinx_copybutton",
    "sphinx_tabs.tabs"
]

# Intersphinx mapping
intersphinx_mapping = {"python": ("https://docs.python.org/3", None)}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True

# Consider enabling numpydoc validation. See:
# https://numpydoc.readthedocs.io/en/latest/validation.html#
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}

# static path
html_static_path = ["_static"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = [".rst", ".md"]

# The master toctree document.
master_doc = "index"



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Entry point."""

from ansys.saf.glow.runtime import glow_main

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution import definition
{% if cookiecutter.with_dash_ui == "yes" %}
from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.ui import app
{% endif %}


def main():
    """Entry point."""

    glow_main(definition, {% if cookiecutter.with_dash_ui == "yes" %} app {% elif cookiecutter.with_dash_ui == "no" %} None {% endif %})


if __name__ == "__main__":
    main()



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.
# CAPTURED v0.4.0 on 6/6/23 from:
# https://github.com/Solution-Applications/common-files/blob/v0.4.0/setup-environment/setup_environment.py

"""
A Python script to automate the setup of the ecosystem of a Python project.

Usage
-----

This script needs to be executed at project's root.

To create a virtual environment and install the dependency management system of the project (if any):
    ``python setup_environment.py``

To override an existing install use the ``-f`` option to force the workspace clean-up:
    ``python setup_environment.py -f``

To select a particular version of the dependency management system use the ``-s`` option:
    ``python setup_environment.py -s 1.3``

To install the production dependencies as well as all the optional dependency groups of the project use the ``-d``
option with the keyword ``all``:
    ``python setup_environment.py -d all``

To install only a particular dependency group use ``run`` for production, ``doc`` for documentation, ``tests`` for
testing, ``style`` for code linting, and ``build`` for build. For example:
    ``python setup_environment.py -d run`` installs production dependencies
    ``python setup_environment.py -d doc`` installs doc dependencies
    ``python setup_environment.py -d tests`` installs tests dependencies
    ``python setup_environment.py -d style`` installs style dependencies
    ``python setup_environment.py -d build`` installs build dependencies

It is also possible to combine several groups:
    ``python setup_environment.py -d run doc tests``

Extra dependency groups refer to any group declared in the configuration file of the dependency management system which
are not part of doc, tests, style and build. To install these groups use the ``x`` option:
    ``python setup_environment.py -x <name-of-the-group>``

Currently, this code only supports:
    * ``windows`` as operating system
    * ``poetry`` and ``flit`` as dependency management systems
    * projects without dependency management systems

There are two locations where the script will search for optional dependencies:
    * First it checks the ``pyproject.toml`` configuration file and search for optional dependency groups
    * Alternatively, it looks for a ``requirements`` folder at project root containing requirements files with the
      name of the dependency group. For instance: requirements/requirements_doc.txt for the documentation group.

The following project structure is expected for projects without a dependency management system:
project-name
├──requirements/                    # Folder containing the optional group of dependencies.
│  ├── requirements_doc.txt         # Requirements file associated to the documentation group.
│  ├── requirements_tests.txt       # Requirements file associated to the tests group.
│  ├── requirements_style.txt       # Requirements file associated to the style group.
│  └── requirements_build.txt       # Requirements file associated to the build group.

The following project structure is expected for projects with a dependency management system:
project-name
└── pyproject.toml                  # Configuration of the build system.

The following lines should be present in the ``pyproject.toml`` file for managing the version of the dependency management system:
Example for using poetry 1.4.0:
[build-system-requirements]
build-system-version = "1.4.0"
"""

# ==================================================== [Imports] ==================================================== #

import argparse
import os
from pathlib import Path
import platform
import re
import shutil
import subprocess
import sys
import textwrap
import time

from packaging.markers import Marker
import toml

# =================================================== [Variables] =================================================== #

supported_dependency_management_systems = {
    "poetry": {
        "configuration_file": "pyproject.toml",
        "build_backend": "poetry.core.masonry.api",
        "required_venv_name": ".venv",
        "lock_file": "poetry.lock",
        "cache_folder": ".poetry\.cache",
        "build_system_venv": ".poetry\.venv",
    },
    "flit": {
        "configuration_file": "pyproject.toml",
        "build_backend": "flit_core.buildapi",
        "required_venv_name": None,
        "lock_file": None,
        "cache_folder": ".flit/.cache",
        "build_system_venv": ".flit/.venv",
    },
}

standard_optional_dependency_groups = ["doc", "tests", "build", "style", "external"]

# =================================================== [Functions] =================================================== #

# Console prints ----------------------------------------------------------------------------------------------------


def print_main_header(text, max_length=100):
    """Display main header."""

    for i in range(max_length):
        print("=", end="")
    print()
    print(text)
    for i in range(max_length):
        print("=", end="")
    print()
    print()


def print_section_header(text, max_length=100):
    """Display a section header in the console."""
    section_header = ""
    if len(text) < max_length:
        section_header = text + " "
        for i in range(len(text), max_length):
            section_header += "-"
    else:
        section_header = text
    print(section_header)
    print()


def print_input_value(input, value, separator=":", separator_position=60):
    """Print input value in console."""
    if len(input) < separator_position:
        text = input
        for i in range(len(text), separator_position):
            text += " "
        text += separator + " " + value
    else:
        text = input + " " + separator + " " + value
    print(text)


def print_inputs_summary(args):
    """Display a summary of the inputs."""

    print(f"OS                                   : {platform.system()}")
    print(f"Python version                       : {get_python_version()}")
    print(f"Virtual environment name             : {args.venv_name}")
    print(f"run dependencies                     : {'yes' if 'run' in args.dependencies else 'no'}")
    for dependency_group in standard_optional_dependency_groups:
        print_input_value(
            f"{dependency_group} dependencies",
            "yes" if dependency_group in args.dependencies else "no",
            separator=":",
            separator_position=37,
        )
    if args.build_system and args.extra_dependencies:
        for dependency_group in args.extra_dependencies:
            print_input_value(
                f"{dependency_group} dependencies",
                "yes",
                separator=":",
                separator_position=37,
            )
    print(f"Dependency management system         : {args.build_system}")
    print(f"Dependency management system version : {args.build_system_version}")
    print(f"Credentials management               : {args.credentials_management_method}")
    print()


# Checks ------------------------------------------------------------------------------------------------------------


def check_dependency_management_system():
    """Check if a dependency management system is available at project root."""
    for dms_name in supported_dependency_management_systems.keys():
        if os.path.exists(supported_dependency_management_systems[dms_name]["configuration_file"]):
            dms_configuration = toml.load(supported_dependency_management_systems[dms_name]["configuration_file"])
            if (
                dms_configuration["build-system"]["build-backend"]
                == supported_dependency_management_systems[dms_name]["build_backend"]
            ):
                return dms_name


def check_virtual_environment_name(args):
    """Check if the virtual environment name is consistent with the build system expectations."""
    if args.build_system:
        if supported_dependency_management_systems[args.build_system]["required_venv_name"]:
            if supported_dependency_management_systems[args.build_system]["required_venv_name"] != args.venv_name:
                old_name = args.venv_name
                args.venv_name = supported_dependency_management_systems[args.build_system]["required_venv_name"]
                print(
                    f"Warning: {args.build_system} expects the name of the virtual environment name to be {args.venv_name}."
                    f"{old_name} is replaced by {args.venv_name}."
                )


def check_python_version(args):
    """
    Check if the version of the current Python interpreter is consistent with the python version specifications
    from the build system.
    """
    if args.build_system:
        # Initialize lower/upper versions
        lower_version, upper_version, lower_bound_symbol, upper_bound_symbol = None, None, None, None
        lower_specification, upper_specification, single_specification = None, None, None
        # Read TOML
        configuration = toml.load(supported_dependency_management_systems[args.build_system]["configuration_file"])
        # Read Python version constraint
        python_compatibility = ""
        if args.build_system == "poetry":
            python_compatibility = configuration["tool"][args.build_system]["dependencies"]["python"].replace(" ", "")
        elif args.build_system == "flit":
            python_compatibility = configuration["project"]["requires-python"].replace(" ", "")
        # Split lower/upper version constraint
        if "," in python_compatibility:
            lower_specification, upper_specification = (
                python_compatibility.split(",")[0],
                python_compatibility.split(",")[1],
            )
        else:
            if python_compatibility.startswith(">"):
                lower_specification, upper_specification = python_compatibility, None
            elif python_compatibility.startswith("<"):
                lower_specification, upper_specification = None, python_compatibility
            else:
                single_specification = python_compatibility
        # Process lower constraint
        if lower_specification:
            lower_version = get_version_from_python_specification(lower_specification)
            lower_bound_symbol = get_sign_from_python_specification(lower_specification)
            marker = Marker(f"python_full_version {lower_bound_symbol} '{lower_version}'")
            if not marker.evaluate():
                raise Exception(f"Python version must be {lower_bound_symbol} {lower_version}.")
        # Process upper constraint
        if upper_specification:
            upper_version = get_version_from_python_specification(upper_specification)
            upper_bound_symbol = get_sign_from_python_specification(upper_specification)
            marker = Marker(f"python_full_version {upper_bound_symbol} '{upper_version}'")
            if not marker.evaluate():
                raise Exception(f"Python version must be {upper_bound_symbol} {upper_version}.")
        # Process single
        if single_specification:
            version = get_version_from_python_specification(single_specification)
            sign = get_sign_from_python_specification(single_specification)
            if sign != "==":
                raise Exception("Unable to interpret python version specification.")
            marker = Marker(f"python_full_version {sign} '{version}'")
            if not marker.evaluate():
                raise Exception(f"Python version must be equal to {version}.")


def check_existing_install(args):
    """Check if an install exists already."""
    return os.path.isdir(args.venv_name)


def check_inputs(args):
    """Check inputs consistency."""
    # Check platform
    if sys.platform != "win32":
        raise Exception("Only Windows operating systems are supported.")
    # Rework dependencies argument if all option is selected
    if "all" in args.dependencies:
        args.install_all = True
        args.dependencies = standard_optional_dependency_groups + ["run"]
    else:
        args.install_all = False
    # Check if a dependency management system is declared
    args.build_system = check_dependency_management_system()
    # Check virtual environment name
    check_virtual_environment_name(args)
    # Check python version
    check_python_version(args)
    # Check if an install already exists
    args.has_install = check_existing_install(args)


# General-purpose ---------------------------------------------------------------------------------------------------


def read_integers_from_string(string):
    """
    Scan a string and extract integers.

    The regex matches any digit character (0-9).
    """
    return re.findall(r"\d+", string)


def remove_scheme_from_url(url):
    """Remove the scheme part of a URL."""
    items = url.split("://")
    if len(items) == 2:
        return items[0], items[1]
    else:
        raise Exception(f"Fail to return the URL without the scheme part for {url}.")


def get_python_version():
    """Get Python version."""
    return f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"


def get_python_package(
    package_name: str,
    package_version: str = "*",
    method: str = "install",
    pypi_url: str = "https://pypi.org/simple",
    private_pypi_token_name: str = "",
    no_deps: bool = False,
    python_executable: str = sys.executable,
    verbose: bool = True,
) -> None:
    """
    Install or download a python package using PIP.

    Parameters
    ----------
    package_name : str
        Name of the package.
    package_version : str
        Version of the package.
    method : str
        Get method: ``download`` or ``install``
    pypi_url : str
        URL of the PyPI server.
    private_pypi_token_name : str
        Name of the environment variable holding the token to access to the PyPI.
    python_executable : str
        Path to the Python executable.

    Returns
    -------
    None
    """

    # Set command
    command = f"{python_executable} -m pip {method} {package_name}"
    # Add package version if specified
    if package_version != "*":
        command += f"=={package_version}"
    # Add part related to private PyPI source
    if pypi_url != "https://pypi.org/simple":
        # Check if token is available
        if private_pypi_token_name is None:
            raise Exception(f"No token specified for private PyPI server {pypi_url}.")
        # Check if the environment variable associated to the private PyPI token exists
        private_pypi_token = os.getenv(private_pypi_token_name)
        if private_pypi_token is None:
            raise Exception(f"Environment variable {private_pypi_token_name} does not exist.")
        # Split absolute URL
        url_scheme, relative_url = remove_scheme_from_url(pypi_url)
        # Update command line
        command += f" -i {url_scheme}://PAT:{private_pypi_token}@{relative_url}"
    # Add no-deps option
    if no_deps:
        command += " --no-deps"
    # Run command with dummy version to force PIP to return the list of released versions
    returncode, stdout, stderr = run_command(command, display_output=verbose)
    if returncode != 0:
        print(stdout)
        raise Exception("Command failed with error.")


def run_command(cmd, display_output=True):
    """
    Run command.
    """
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    if display_output:
        stdout, stderr = b"", b""
        if process.stdout is not None:
            for line in process.stdout:
                sys.stdout.buffer.write(line)
                sys.stdout.buffer.flush()
                stdout += line
        process.wait()
    else:
        stdout, stderr = process.communicate()

    if isinstance(stdout, bytes):
        try:
            stdout = stdout.decode().splitlines()
        except:
            raise Exception("Unable to decode stdout.")

    if isinstance(stderr, bytes):
        try:
            stderr = stderr.decode().splitlines()
        except:
            raise Exception("Unable to decode stderr.")

    return process.returncode, stdout, stderr


def upgrade_pip(python_executable: str = sys.executable) -> None:
    """Upgrade to latest PIP version in the virtual environment"""
    print("Upgrade to latest pip version")
    subprocess.run(
        [python_executable, "-m", "pip", "install", "--upgrade", "pip"],
        check=True,
        shell=True,
    )
    print()


def create_virtual_environment(args, venv: str = ".venv"):
    """Create a virtual environment."""

    print("Create virtual environment")
    if not args.has_install or args.force_clear or args.force_clear_all:
        subprocess.run([sys.executable, "-m", "venv", venv], check=True, shell=True)
    else:
        print("Skipped")
    print()


# Dependency Management System (Build System) -----------------------------------------------------------------------


def get_private_sources(configuration_file):
    """Get list of private sources from configuration file."""
    configuration = toml.load(configuration_file)
    try:
        private_sources = configuration["tool"]["poetry"]["source"]
    except:
        private_sources = []
    return private_sources


def get_version_from_python_specification(specification):
    """Read the version."""
    return ".".join(read_integers_from_string(specification))


def get_sign_from_python_specification(specification):
    """Read the mathematical symbol expressing a constraint on the version."""
    first_part = specification.split(".")[0]
    sign = "".join([i for i in first_part if not i.isdigit()])
    if sign == "":
        sign = "=="
    return sign


def install_build_system(args):
    """Install build system."""
    print("Install dependency management system.")
    if args.build_system:
        if not args.has_install or args.force_clear or args.force_clear_all:
            configuration = toml.load(supported_dependency_management_systems[args.build_system]["configuration_file"])
            if (
                args.build_system_version == "*"
                and "build-system-version" in configuration["build-system-requirements"]
            ):
                args.build_system_version = configuration["build-system-requirements"]["build-system-version"]
            if args.build_system == "poetry":
                create_virtual_environment(
                    args, venv=supported_dependency_management_systems[args.build_system]["build_system_venv"]
                )
                upgrade_pip(
                    python_executable=rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\python"
                )
                get_python_package(
                    args.build_system,
                    args.build_system_version,
                    method="install",
                    python_executable=rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\python",
                    verbose=args.verbose,
                )
            else:
                get_python_package(
                    args.build_system,
                    args.build_system_version,
                    method="install",
                    python_executable=rf".\{args.venv_name}\Scripts\python",
                    verbose=args.verbose,
                )
            print()
            # Create a file symbolic link from the virtual environment (.venv) that the build system (poetry) manages
            # to the build system executable (poetry.exe) in the poetry virtual environment (.poetry/.venv).  This
            # ensures that the correct poetry is accessible when the managed virtual environment is activated
            build_system_executable = rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\{args.build_system}.exe"
            if os.path.exists(build_system_executable):
                subprocess.run(
                    [
                        "powershell",
                        "-Command",
                        "New-Item",
                        "-ItemType",
                        "SymbolicLink",
                        "-Path",
                        rf".\{args.venv_name}\Scripts\{args.build_system}.exe",
                        "-Target",
                        build_system_executable,
                    ]
                )
            return
    print("Skipped")
    print()


def configure_build_system(args):
    """Configure the build system to enable connection to private sources."""
    print("Configure dependency management system.")
    if args.build_system:
        if not args.has_install or args.force_clear or args.force_clear_all:
            if args.build_system == "poetry":
                configure_poetry(
                    supported_dependency_management_systems[args.build_system]["build_system_venv"],
                    args.credentials_management_method,
                )
                print()
            return
    print("Skipped")
    print()


def configure_poetry(venv_name, credentials_management_method):
    """Configure Poetry."""
    # Get list of private sources
    private_sources = get_private_sources("pyproject.toml")
    # Delete existing configuration
    print("Clean-up existing poetry configurations")
    if sys.platform == "win32":
        username = os.getlogin()
        path_poetry_config = rf"C:\Users\{username}\AppData\Roaming\pypoetry"
        if os.path.isfile(os.path.join(path_poetry_config, "config.toml")):
            os.remove(os.path.join(path_poetry_config, "config.toml"))
        if os.path.isfile(os.path.join(path_poetry_config, "auth.toml")):
            os.remove(os.path.join(path_poetry_config, "auth.toml"))
    # Turn-on in-project
    subprocess.run([rf".\{venv_name}\Scripts\poetry", "config", "virtualenvs.in-project", "true"], check=True)
    # Turn-on poetry cache
    subprocess.run(
        [
            rf".\{venv_name}\Scripts\poetry",
            "config",
            "cache-dir",
            supported_dependency_management_systems["poetry"]["cache_folder"],
        ],
        check=True,
    )
    # Declare credentials for private sources
    for source in private_sources:
        print(f"Declare credentials for {source['name']}")
        # Get source PAT
        if source["name"].lower() == "pypi":
            continue
        elif source["url"] == "https://pkgs.dev.azure.com/pyansys/_packaging/pyansys/pypi/simple/":
            token = os.environ["PYANSYS_PRIVATE_PYPI_PAT"]
        elif source["url"] == "https://pkgs.dev.azure.com/pyansys/_packaging/ansys-solutions/pypi/simple/":
            token = os.environ["SOLUTIONS_PRIVATE_PYPI_PAT"]
        else:
            raise Exception(f"Unknown private source {source['name']} with url {source['url']}.")
        # Store credentials
        if credentials_management_method == "keyring":
            # Declare source URL
            command_line = f"{venv_name}/Scripts/poetry config repositories.{source['name']} {source['url']}"
            subprocess.run(command_line, check=True)
            # Declare source credentials
            command_line = f"{venv_name}/Scripts/poetry config http-basic.{source['name']} PAT {token}"
            subprocess.run(command_line, check=True)
        elif credentials_management_method == "environment-variables":
            # Format source name to comply with Poetry environment variable syntax
            source_name = source["name"].upper().replace("-", "_")
            # Create Poetry environment variable
            os.environ[f"POETRY_HTTP_BASIC_{source_name}_USERNAME"] = "PAT"
            os.environ[f"POETRY_HTTP_BASIC_{source_name}_PASSWORD"] = token


def check_dependency_group(dependency_group, configuration):
    """Return True if the dependency group is available in the configuration file."""
    try:
        configuration["tool"]["poetry"]["group"][dependency_group]
        return True
    except:
        return False

def configure_portal_database(venv_name):
    subprocess.run(
        [
            rf".\{venv_name}\Scripts\poetry",
            "run",
            "configure-portal-database",
        ],
        check=True,
    )

# Specifics ---------------------------------------------------------------------------------------------------------


def parser():
    """Parse command line arguments."""
    # Code Name
    program_name = "Setup Environment"
    # Code description
    program_description = "A Python script to setup the Python ecosystem of a solution or framework."
    # Create top-level parser
    parser = argparse.ArgumentParser(
        prog=program_name,
        usage=None,
        prefix_chars="-",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=textwrap.dedent(program_description),
    )
    parser._action_groups.pop()
    # Definition of the group of arguments
    optional_inputs = parser.add_argument_group("Optional arguments")
    # Optional parameters
    optional_inputs.add_argument(
        "-d",
        "--dependencies",
        type=str,
        nargs="+",
        help="List of dependency groups to be installed separated by space. Example: run tests doc style build",
        default=[],
        choices=standard_optional_dependency_groups + ["run", "all"],
        required=False,
    )
    optional_inputs.add_argument(
        "-x",
        "--extra-dependencies",
        type=str,
        nargs="+",
        help="List of extra dependency groups to be installed separated by space.",
        default=[],
        required=False,
    )
    optional_inputs.add_argument(
        "-s",
        "--build-system-version",
        type=str,
        help="Build system version",
        default="1.5.1",
        required=False,
    )
    optional_inputs.add_argument(
        "-c",
        "--credentials-management-method",
        type=str,
        help="Method to store private sources credentials.",
        default="keyring",
        choices=["keyring", "environment-variables"],
        required=False,
    )
    optional_inputs.add_argument(
        "-e",
        "--venv-name",
        type=str,
        help="Name of the virtual environment. Must be '.venv' when using poetry",
        default=".venv",
        required=False,
    )
    optional_inputs.add_argument(
        "-F",
        "--force-clear-all",
        help="Clean-up the workspace. Delete existing .venv, .poetry\.venv, .poetry\.cache and poetry.lock.",
        action="store_true",
        required=False,
    )
    optional_inputs.add_argument(
        "-f",
        "--force-clear",
        help="Clean-up the workspace. Delete existing .venv., .poetry\.venv and .poetry\.cache.",
        action="store_true",
        required=False,
    )
    optional_inputs.add_argument(
        "-v",
        "--verbose",
        help="Activate verbose mode.",
        action="store_true",
        required=False,
    )

    return parser.parse_args()


def clear_workspace(args):
    """Remove residual items form previous installation (like venv directory, lock file ...)."""
    print("Clear workspace")
    if args.force_clear or args.force_clear_all:
        # Remove virtual environment
        if os.path.isdir(args.venv_name):
            print(f"Delete existing virtual environment {args.venv_name}.")
            shutil.rmtree(args.venv_name)
        # Remove poetry virtual environment
        if os.path.isdir(supported_dependency_management_systems[args.build_system]["build_system_venv"]):
            print("Delete existing poetry virtual environment")
            shutil.rmtree(supported_dependency_management_systems[args.build_system]["build_system_venv"])
        # Remove poetry cache
        if os.path.isdir(supported_dependency_management_systems[args.build_system]["cache_folder"]):
            print("Delete existing poetry cache")
            shutil.rmtree(supported_dependency_management_systems[args.build_system]["cache_folder"])
        if args.force_clear_all:
            # Remove lock file
            if args.build_system:
                if supported_dependency_management_systems[args.build_system]["lock_file"]:
                    if os.path.isfile(supported_dependency_management_systems[args.build_system]["lock_file"]):
                        print("Delete existing poetry lock file")
                        os.remove(supported_dependency_management_systems[args.build_system]["lock_file"])
    else:
        print("Skipped")
    print()


def install_production_dependencies(args):
    """Install the package (mandatory requirements only)."""
    print("Install production dependencies")
    if args.build_system and "run" in args.dependencies:
        if args.build_system == "poetry":
            subprocess.run(
                [
                    rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\poetry",
                    "install",
                    "-vv",
                ],
                check=True,
                shell=True,
            )
        elif args.build_system == "flit":
            subprocess.run([rf".\{args.venv_name}\Scripts\flit", "install"], check=True, shell=True)
        print()
        return
    print("Skipped")
    print()


def install_optional_dependencies(args):
    """Install optional requirements (doc, tests, build or style)."""
    # Load configuration file if a dependency management system is available
    if args.build_system:
        configuration = toml.load(supported_dependency_management_systems[args.build_system]["configuration_file"])
    else:
        configuration = {}
    # Install standard optional dependency groups
    for dependency_group in standard_optional_dependency_groups:
        print(f"Install {dependency_group} dependencies")
        if dependency_group in args.dependencies:
            # Install dependency group in the configuration file of the build system
            if args.build_system:
                has_dependency_group = check_dependency_group(dependency_group, configuration)
                if has_dependency_group:
                    print("Installing from build system configuration file.")
                    subprocess.run(
                        [
                            rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\poetry",
                            "install",
                            "--only",
                            dependency_group,
                            "-vv",
                        ],
                        check=True,
                        shell=True,
                    )
                    print()
                    continue
            # Alternatively search dependency group in the requirements folder
            requirements_file = os.path.join("requirements", f"requirements_{dependency_group}.txt")
            if os.path.exists(requirements_file):
                print("Installing from requirements file.")
                subprocess.run(
                    [
                        rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\python",
                        "-m",
                        "poetry",
                        "run",
                        "pip",
                        "install",
                        "-r",
                        requirements_file,
                        "--no-warn-conflicts",
                    ],
                    check=True,
                    shell=True,
                )
                print()
                continue
        print("Skipped")
        print()
    # Install extra optional dependency groups
    if args.build_system and args.extra_dependencies:
        for dependency_group in args.extra_dependencies:
            print(f"Install {dependency_group} dependencies")
            has_dependency_group = check_dependency_group(dependency_group, configuration)
            if has_dependency_group:
                subprocess.run(
                    [
                        rf".\{supported_dependency_management_systems[args.build_system]['build_system_venv']}\Scripts\poetry",
                        "install",
                        "--only",
                        dependency_group,
                        "-vv",
                    ],
                    check=True,
                    shell=True,
                )
            else:
                print(f"No dependency group named {dependency_group}.")
                print("Skipped")
            print()


def main():
    """Sequence of operations leading to the complete Python ecosystem."""

    # Start timer
    time_on = time.time()
    # Get current working directory
    working_directory = os.getcwd()
    # Get installation directory
    install_directory = Path(__file__).parent.absolute()
    # Move to install directory
    os.chdir(install_directory)

    # Initialize session ----------------------------------------------------------------------------------------------

    # Read command line inputs
    args = parser()

    # Check inputs consistency
    check_inputs(args)

    # Display header
    print_main_header("Setup Environment")

    # Display inputs summary
    print_inputs_summary(args)

    # Clear workspace -------------------------------------------------------------------------------------------------

    print_section_header("Clear workspace", max_length=100)

    clear_workspace(args)

    # Setup virtual environment ---------------------------------------------------------------------------------------

    print_section_header("Setup virtual environment", max_length=100)

    create_virtual_environment(args)

    upgrade_pip(python_executable=rf".\{args.venv_name}\Scripts\python")

    # Setup dependency management system ------------------------------------------------------------------------------

    print_section_header("Setup dependency management system", max_length=100)

    install_build_system(args)

    configure_build_system(args)

    # Install dependencies --------------------------------------------------------------------------------------------

    print_section_header("Install dependencies", max_length=100)

    install_production_dependencies(args)

    install_optional_dependencies(args)

    # Back to current working directory
    os.chdir(working_directory)

    # Configure database
    if 'run' in args.dependencies:
        configure_portal_database(args.venv_name)

    # Compute execution time
    elapsed_time = (time.time() - time_on) / 60  # in minutes

    print("You are all set!")
    print("Navigate to project root and activate your environment with one these commands:")
    print(rf"   - For Windows CMD       : {args.venv_name}\Scripts\activate.bat")
    print(rf"   - For Windows Powershell: {args.venv_name}\Scripts\Activate.ps1")
    print("Enjoy!")
    print()
    print("Execution time: %.1f minutes." % (elapsed_time))
    print()


# =================================================== [Execution] =================================================== #

if __name__ == "__main__":
    main()



"""Sphinx documentation configuration file."""
from datetime import datetime
import os

from ansys_sphinx_theme import get_version_match
{%- if cookiecutter.__logo == "ansys" and cookiecutter.__logo_color == "white" %}
from ansys_sphinx_theme import ansys_logo_white as logo
{%- elif cookiecutter.__logo == "ansys" and cookiecutter.__logo_color == "black" %}
from ansys_sphinx_theme import ansys_logo_black as logo
{%- elif cookiecutter.__logo == "pyansys" and cookiecutter.__logo_color == "white" %}
from ansys_sphinx_theme import pyansys_logo_white as logo
{%- elif cookiecutter.__logo == "pyansys" and cookiecutter.__logo_color == "black" %}
from ansys_sphinx_theme import pyansys_logo_black as logo
{%- endif %}

{%- if cookiecutter.__template_name != "doc-project" %}
from {{cookiecutter.__pkg_namespace}} import __version__
{%- endif %}


# Project information
{%- if cookiecutter.__template_name in ["doc-project"] %}
project = "{{ cookiecutter.__project_name_slug }}"
{%- else %}
project = "{{ cookiecutter.__pkg_name }}"
{%- endif %}
copyright = f"(c) {datetime.now().year} ANSYS, Inc. All rights reserved"
author = "ANSYS, Inc."
{%- if cookiecutter.__template_name != "doc-project" %}
release = version = __version__
{%- elif cookiecutter.__template_name == "doc-project" %}
release = version = "{{ cookiecutter.__version }}"
{%- endif %}
cname = os.getenv("DOCUMENTATION_CNAME", "docs.pyansys.com")

# Select desired logo, theme, and declare the html title
html_logo = logo
html_theme = "ansys_sphinx_theme"
html_short_title = html_title = "{{ cookiecutter.__project_name_slug }}"

# specify the location of your github repo
html_theme_options = {
    "github_url": "{{ cookiecutter.__repository_url }}",
    "show_prev_next": False,
    "show_breadcrumbs": True,
    "additional_breadcrumbs": [
        {%- if cookiecutter.__logo == "ansys" %}
        ("Ansys", "https://dev.docs.ansys.com/"),
        {%- elif cookiecutter.__logo == "pyansys" %}
        ("PyAnsys", "https://docs.pyansys.com/"),
        {%- endif %}
    ],
    "switcher": {
        "json_url": f"https://{cname}/versions.json",
        "version_match": get_version_match(__version__),
    },
    "check_switcher": False,
}

# Sphinx extensions
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "numpydoc",
    "sphinx.ext.intersphinx",
    "sphinx_copybutton",
]

# Intersphinx mapping
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    # kept here as an example
    # "scipy": ("https://docs.scipy.org/doc/scipy/reference", None),
    # "numpy": ("https://numpy.org/devdocs", None),
    # "matplotlib": ("https://matplotlib.org/stable", None),
    # "pandas": ("https://pandas.pydata.org/pandas-docs/stable", None),
    # "pyvista": ("https://docs.pyvista.org/", None),
    # "grpc": ("https://grpc.github.io/grpc/python/", None),
}

# numpydoc configuration
numpydoc_show_class_members = False
numpydoc_xref_param_type = True

# Consider enabling numpydoc validation. See:
# https://numpydoc.readthedocs.io/en/latest/validation.html#
numpydoc_validate = True
numpydoc_validation_checks = {
    "GL06",  # Found unknown section
    "GL07",  # Sections are in the wrong order.
    "GL08",  # The object does not have a docstring
    "GL09",  # Deprecation warning should precede extended summary
    "GL10",  # reST directives {directives} must be followed by two colons
    "SS01",  # No summary found
    "SS02",  # Summary does not start with a capital letter
    # "SS03", # Summary does not end with a period
    "SS04",  # Summary contains heading whitespaces
    # "SS05", # Summary must start with infinitive verb, not third person
    "RT02",  # The first line of the Returns section should contain only the
    # type, unless multiple values are being returned"
}


# static path
html_static_path = ["_static"]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# The suffix(es) of source filenames.
source_suffix = ".rst"

# The master toctree document.
master_doc = "index"



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Backend of the second step."""


from ansys.saf.glow.solution import StepModel


class SecondStep(StepModel):
    """Step definition of the second step."""

    id: str = "hey"



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the second step."""

from dash import dash_table
from dash_extensions.enrich import html
import pandas as pd

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep


class DesignTable(object):
    """"""

    def __init__(self, data):
        """Constructor."""

        self.data = data

    def render(self):
        """Generate table."""

        data = self.data.copy()

        for col, dtype in data.dtypes.items():
            if dtype == "bool":
                data[col] = data[col].astype("str")

        data = data.round(6)

        return dash_table.DataTable(
            data=data.to_dict("records"),
            columns=[{"name": i, "id": i, "type": "text"} for i in data.columns],
            fixed_rows={"headers": True},
            sort_action="native",
            row_selectable="multi",
            page_action="native",
            style_header={"textAlign": "center", "font_family": "Roboto", "font_size": "15px", "fontWeight": "bold"},
            style_cell={
                "textAlign": "center",
                "font_family": "Roboto",
                "font_size": "15px",
            },
            style_data_conditional=[
                {
                    "if": {"column_id": "Status", "filter_query": '{Status} eq "Succeeded"'},
                    "backgroundColor": "rgb(223, 240, 208)",
                    "color": "rgb(0, 0, 0)",
                },
                {
                    "if": {"column_id": "Status", "filter_query": '{Status} eq "Not succeeded"'},
                    "backgroundColor": "rgb(254, 221, 215)",
                    "color": "rgb(0, 0, 0)",
                },
                {
                    "if": {"column_id": "Feasible", "filter_query": '{Feasible} eq "True"'},
                    "backgroundColor": "rgb(223, 240, 208)",
                    "color": "rgb(0, 0, 0)",
                },
                {
                    "if": {"column_id": "Feasible", "filter_query": '{Feasible} eq "False"'},
                    "backgroundColor": "rgb(254, 221, 215)",
                    "color": "rgb(0, 0, 0)",
                },
            ],
        )


def layout(monitoring_step: MonitoringStep):
    """Layout of the second step UI."""

    monitoring_step.get_design_table()

    design_table = DesignTable(pd.DataFrame(monitoring_step.design_table))

    return html.Div([design_table.render()])



# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the node status overview tab."""

from dash_extensions.enrich import html
import optislang_dash_lib

from ansys.solutions.{{ cookiecutter.__solution_name_slug }}.solution.monitoring_step import MonitoringStep


def layout(monitoring_step: MonitoringStep):
    """Layout of the node status overview tab."""

    monitoring_step.get_project_state() # fetches data on page load

    return html.Div(
        id="node-status-view",
        children=[
            optislang_dash_lib.Nodestatusviewcomponent(
                id="node-status-view-component",
                project_state=monitoring_step.project_state,
            ),
        ],
    )